@@ -3,7 +3,7 @@
 {
     'name': 'Barcode - GS1 Nomenclature',
     'version': '1.0',
-    'category': 'Hidden',
+    'category': 'Supply Chain/Inventory',
     'summary': 'Parse barcodes according to the GS1-128 specifications',
     'depends': ['barcodes', 'uom'],
     'data': [
@@ -11,14 +11,15 @@
         'views/barcodes_view.xml',
     ],
     'installable': True,
-    'auto_install': False,
     'assets': {
         'web.assets_backend': [
             'barcodes_gs1_nomenclature/static/src/js/barcode_parser.js',
+            'barcodes_gs1_nomenclature/static/src/js/barcode_service.js',
         ],
-        'web.qunit_suite_tests': [
+        'web.assets_unit_tests': [
             'barcodes_gs1_nomenclature/static/src/js/tests/**/*',
         ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
