@@ -9,10 +9,20 @@ Import a custom data module
 This module allows authorized users to import a custom data module (.xml files and static assests)
 for customization purpose.
 """,
-    'category': 'Tools',
+    'category': 'Hidden/Tools',
     'depends': ['web'],
     'installable': True,
-    'auto_install': False,
-    'data': ['views/base_import_module_view.xml'],
+    'auto_install': True,
+    'data': [
+        'security/ir.model.access.csv',
+        'views/base_import_module_view.xml',
+        'views/ir_module_views.xml',
+    ],
+    'assets': {
+        'web.assets_backend': [
+            'base_import_module/static/src/**/*',
+        ]
+    },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
