@@ -8,10 +8,13 @@ from . import test_crm_lead_notification
 from . import test_crm_lead_convert
 from . import test_crm_lead_convert_mass
 from . import test_crm_lead_duplicates
-from . import test_crm_lead_lost
 from . import test_crm_lead_merge
 from . import test_crm_lead_multicompany
 from . import test_crm_lead_smart_calendar
 from . import test_crm_ui
 from . import test_crm_pls
+from . import test_crm_rainbowman
+from . import test_digest
 from . import test_performances
+from . import test_res_partner
+from . import test_sales_team_ui
