@@ -28,9 +28,7 @@
         'web.assets_backend': [
             'crm_iap_mine/static/src/js/**/*',
         ],
-        'web.assets_qweb': [
-            'crm_iap_mine/static/src/xml/**/*',
-        ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
