@@ -6,7 +6,6 @@
     'version': '1.1',
     'category': 'Sales/CRM',
     'summary': 'Add SMS capabilities to CRM',
-    'description': "",
     'depends': ['crm', 'sms'],
     'data': [
         'views/crm_lead_views.xml',
@@ -14,7 +13,7 @@
         'security/sms_security.xml',
     ],
     'installable': True,
-    'application': False,
     'auto_install': True,
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
