@@ -21,8 +21,9 @@ Badge received are displayed on the user profile.
     'auto_install': True,
     'assets': {
         'web.assets_backend': [
-            'hr_gamification/static/**/*',
+            'hr_gamification/static/src/**/*',
         ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
