@@ -3,10 +3,10 @@
 
 {
     'name': 'Time Off',
-    'version': '1.5',
+    'version': '1.6',
     'category': 'Human Resources/Time Off',
     'sequence': 85,
-    'summary': 'Allocate PTOs and follow leaves requests',
+    'summary': 'Allocate time off and follow leave requests',
     'website': 'https://www.odoo.com/app/time-off',
     'description': """
 Manage time off requests and allocations
@@ -27,22 +27,29 @@ A synchronization with an internal agenda (Meetings of the CRM module) is also p
     'depends': ['hr', 'calendar', 'resource'],
     'data': [
         'data/report_paperformat.xml',
-        'data/mail_data.xml',
-        'data/hr_holidays_data.xml',
+        'data/mail_activity_type_data.xml',
+        'data/mail_message_subtype_data.xml',
+        'data/ir_attachment_data.xml',
+        'data/hr_leave_type_data.xml',
         'data/ir_cron_data.xml',
+        'data/hr_holidays_tour.xml',
 
         'security/hr_holidays_security.xml',
         'security/ir.model.access.csv',
 
+        'wizard/hr_holidays_cancel_leave_views.xml',
+        'wizard/hr_holidays_summary_employees_views.xml',
+        'wizard/hr_leave_generate_multi_wizard_views.xml',
+        'wizard/hr_leave_allocation_generate_multi_wizard_views.xml',
+
         'views/resource_views.xml',
         'views/hr_leave_views.xml',
         'views/hr_leave_type_views.xml',
         'views/hr_leave_allocation_views.xml',
         'views/hr_leave_accrual_views.xml',
+        'views/hr_leave_mandatory_day_views.xml',
         'views/mail_activity_views.xml',
-
-        'wizard/hr_holidays_summary_employees_views.xml',
-        'wizard/hr_departure_wizard_views.xml',
+        'views/calendar_views.xml',
 
         'report/hr_holidays_templates.xml',
         'report/hr_holidays_reports.xml',
@@ -58,33 +65,33 @@ A synchronization with an internal agenda (Meetings of the CRM module) is also p
     ],
     'installable': True,
     'application': True,
-    'auto_install': False,
     'assets': {
-        'mail.assets_discuss_public': [
-            'hr_holidays/static/src/components/*/*',
-            'hr_holidays/static/src/models/*/*.js',
-        ],
         'web.assets_backend': [
-            'hr_holidays/static/src/js/time_off_calendar.js',
-            'hr_holidays/static/src/js/time_off_calendar_employee.js',
-            'hr_holidays/static/src/js/radio_image.js',
-            'hr_holidays/static/src/js/leave_stats_widget.js',
-            'hr_holidays/static/src/models/*/*.js',
-            'hr_holidays/static/src/scss/time_off.scss',
-            'hr_holidays/static/src/components/*/*.scss',
-            'hr_holidays/static/src/scss/accrual_plan_level.scss',
+            'hr_holidays/static/src/**/*',
+            ('remove', 'hr_holidays/static/src/views/graph/**'),
+            # Don't include dark mode files in light mode
+            ('remove', 'hr_holidays/static/src/**/*.dark.scss'),
         ],
-        'web.tests_assets': [
-            'hr_holidays/static/tests/helpers/**/*',
+        'web.assets_backend_lazy': [
+            'hr_holidays/static/src/views/graph/**',
         ],
-        'web.qunit_suite_tests': [
-            'hr_holidays/static/src/components/*/tests/*.js',
-            'hr_holidays/static/tests/test_leave_stats_widget.js',
+        "web.assets_web_dark": [
+            'hr_holidays/static/src/**/*.dark.scss',
         ],
-        'web.assets_qweb': [
-            'hr_holidays/static/src/components/*/*.xml',
-            'hr_holidays/static/src/xml/*.xml',
+        'im_livechat.assets_embed_core': [
+            'hr_holidays/static/src/core/common/**/*',
+        ],
+        'mail.assets_public': [
+            'hr_holidays/static/src/core/common/**/*',
+        ],
+        'web.assets_unit_tests': [
+            'hr_holidays/static/tests/**/*',
+            ('remove', 'hr_holidays/static/tests/tours/**/*'),
+        ],
+        'web.assets_tests': [
+            '/hr_holidays/static/tests/tours/**/*'
         ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
