@@ -9,9 +9,6 @@ class ResConfigSettings(models.TransientModel):
 
     l10n_eu_oss_eu_country = fields.Boolean('Is European country?', compute='_compute_l10n_eu_oss_european_country')
 
-    def refresh_eu_tax_mapping(self):
-        self.env.companies._map_eu_taxes()
-
     @api.depends('company_id')
     def _compute_l10n_eu_oss_european_country(self):
         european_countries = self.env.ref('base.europe').country_ids
