@@ -10,6 +10,7 @@ class ResPartner(models.Model):
     first_name = fields.Char("First Name")
     middle_name = fields.Char("Middle Name")
     last_name = fields.Char("Last Name")
+    l10n_ph_rdo = fields.Char("RDO", help="Revenue District Office")
 
     @api.model
     def _commercial_fields(self):
@@ -20,6 +21,6 @@ class ResPartner(models.Model):
         for partner in self:
             branch_code = '000'
             if partner.country_id.code == 'PH' and partner.vat:
-                match = partner.__check_vat_ph_re.match(partner.vat)
+                match = partner._check_vat_ph_re.match(partner.vat)
                 branch_code = match and match.group(1) and match.group(1)[1:] or branch_code
             partner.branch_code = branch_code
