@@ -1,17 +1,20 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo import _, fields, models
+from markupsafe import Markup
+
+from odoo import _, api, Command, fields, models, tools
 from odoo.exceptions import UserError
 
 
-class MassMailingList(models.Model):
+class MailingList(models.Model):
     """Model of a contact list. """
     _name = 'mailing.list'
     _order = 'name'
     _description = 'Mailing List'
     _mailing_enabled = True
-    # As this model has his own data merge, avoid to enable the generic data_merge on that model.
+    _order = 'create_date DESC'
+    # As this model has their own data merge, avoid to enable the generic data_merge on that model.
     _disable_data_merge = True
 
     name = fields.Char(string='Mailing List', required=True)
@@ -24,21 +27,27 @@ class MassMailingList(models.Model):
     contact_pct_blacklisted = fields.Float(compute="_compute_mailing_list_statistics", string="Percentage of Blacklisted")
     contact_pct_bounce = fields.Float(compute="_compute_mailing_list_statistics", string="Percentage of Bouncing")
     contact_ids = fields.Many2many(
-        'mailing.contact', 'mailing_contact_list_rel', 'list_id', 'contact_id',
+        'mailing.contact', 'mailing_subscription', 'list_id', 'contact_id',
         string='Mailing Lists', copy=False)
-    mailing_count = fields.Integer(compute="_compute_mailing_list_count", string="Number of Mailing")
-    mailing_ids = fields.Many2many('mailing.mailing', 'mail_mass_mailing_list_rel', string='Mass Mailings', copy=False)
+    mailing_count = fields.Integer(compute="_compute_mailing_count", string="Number of Mailing")
+    mailing_ids = fields.Many2many(
+        'mailing.mailing', 'mail_mass_mailing_list_rel',
+        string='Mass Mailings', copy=False)
     subscription_ids = fields.One2many(
-        'mailing.contact.subscription', 'list_id', string='Subscription Information',
+        'mailing.subscription', 'list_id',
+        string='Subscription Information',
         copy=True, depends=['contact_ids'])
-    is_public = fields.Boolean(default=True, help="The mailing list can be accessible by recipient in the unsubscription"
-                                                  " page to allows him to update his subscription preferences.")
+    is_public = fields.Boolean(
+        string='Show In Preferences', default=False,
+        help='The mailing list can be accessible by recipients in the subscription '
+             'management page to allow them to update their preferences.')
 
     # ------------------------------------------------------
     # COMPUTE / ONCHANGE
     # ------------------------------------------------------
 
-    def _compute_mailing_list_count(self):
+    @api.depends('mailing_ids')
+    def _compute_mailing_count(self):
         data = {}
         if self.ids:
             self.env.cr.execute('''
@@ -50,12 +59,15 @@ class MassMailingList(models.Model):
         for mailing_list in self:
             mailing_list.mailing_count = data.get(mailing_list._origin.id, 0)
 
+    @api.depends('contact_ids')
     def _compute_mailing_list_statistics(self):
         """ Computes various statistics for this mailing.list that allow users
         to have a global idea of its quality (based on blacklist, opt-outs, ...).
 
         As some fields depend on the value of each other (mainly percentages),
         we compute everything in a single method. """
+        # flush, notably to have email_normalized computed on contact model
+        self.env.flush_all()
 
         # 1. Fetch contact data and associated counts (total / blacklist / opt-out)
         contact_statistics_per_mailing = self._fetch_contact_statistics()
@@ -66,13 +78,13 @@ class MassMailingList(models.Model):
         bounce_per_mailing = {}
         if self.ids:
             sql = '''
-                SELECT mclr.list_id, COUNT(DISTINCT mc.id)
+                SELECT list_sub.list_id, COUNT(DISTINCT mc.id)
                 FROM mailing_contact mc
-                LEFT OUTER JOIN mailing_contact_list_rel mclr
-                ON mc.id = mclr.contact_id
+                LEFT OUTER JOIN mailing_subscription list_sub
+                ON mc.id = list_sub.contact_id
                 WHERE mc.message_bounce > 0
-                AND mclr.list_id in %s
-                GROUP BY mclr.list_id
+                AND list_sub.list_id in %s
+                GROUP BY list_sub.list_id
             '''
             self.env.cr.execute(sql, (tuple(self.ids),))
             bounce_per_mailing = dict(self.env.cr.fetchall())
@@ -108,22 +120,51 @@ class MassMailingList(models.Model):
             if mass_mailings > 0:
                 raise UserError(_("At least one of the mailing list you are trying to archive is used in an ongoing mailing campaign."))
 
-        return super(MassMailingList, self).write(vals)
+        return super().write(vals)
 
-    def name_get(self):
-        return [(list.id, "%s (%s)" % (list.name, list.contact_count)) for list in self]
+    @api.depends('contact_count')
+    def _compute_display_name(self):
+        for mailing_list in self:
+            mailing_list.display_name = f"{mailing_list.name} ({mailing_list.contact_count})"
 
-    def copy(self, default=None):
-        self.ensure_one()
-
-        default = dict(default or {},
-                       name=_('%s (copy)', self.name),)
-        return super(MassMailingList, self).copy(default)
+    def copy_data(self, default=None):
+        vals_list = super().copy_data(default=default)
+        return [dict(vals, name=self.env._("%s (copy)", mailing_list.name)) for mailing_list, vals in zip(self, vals_list)]
 
     # ------------------------------------------------------
     # ACTIONS
     # ------------------------------------------------------
 
+    def action_open_import(self):
+        """Open the mailing list contact import wizard."""
+        action = self.env['ir.actions.actions']._for_xml_id('mass_mailing.mailing_contact_import_action')
+        action['context'] = {
+            **self.env.context,
+            'default_mailing_list_ids': self.ids,
+            'default_subscription_ids': [
+                Command.create({'list_id': mailing_list.id})
+                for mailing_list in self
+            ],
+        }
+        return action
+
+    def action_send_mailing(self):
+        """Open the mailing form view, with the current lists set as recipients."""
+        action = self.env["ir.actions.actions"]._for_xml_id('mass_mailing.mailing_mailing_action_mail')
+
+        action.update({
+            'context': {
+                **self.env.context,
+                'default_contact_list_ids': self.ids,
+                'default_mailing_type': 'mail',
+                'default_model_id': self.env['ir.model']._get_id('mailing.list'),
+            },
+            'target': 'current',
+            'view_type': 'form',
+        })
+
+        return action
+
     def action_view_contacts(self):
         action = self.env["ir.actions.actions"]._for_xml_id("mass_mailing.action_view_mass_mailing_contacts")
         action['domain'] = [('list_ids', 'in', self.ids)]
@@ -165,7 +206,7 @@ class MassMailingList(models.Model):
             mailing list in 'self'. Possibility to archive the mailing lists
             'src_lists' after the merge except the destination mailing list 'self'.
         """
-        # Explation of the SQL query with an example. There are the following lists
+        # Explanation of the SQL query with an example. There are the following lists
         # A (id=4): yti@odoo.com; yti@example.com
         # B (id=5): yti@odoo.com; yti@openerp.com
         # C (id=6): nothing
@@ -179,17 +220,16 @@ class MassMailingList(models.Model):
         #           5 | yti@example.com           |          1 |        4 |
         #           7 | yti@openerp.com           |          1 |        5 |
         #
-        # The row_column is kind of an occurence counter for the email address.
+        # The row_column is kind of an occurrence counter for the email address.
         # Then we create the Many2many relation between the destination list and the contacts
         # while avoiding to insert an existing email address (if the destination is in the source
         # for example)
         self.ensure_one()
         # Put destination is sources lists if not already the case
         src_lists |= self
-        self.env['mailing.contact'].flush(['email', 'email_normalized'])
-        self.env['mailing.contact.subscription'].flush(['contact_id', 'opt_out', 'list_id'])
+        self.env.flush_all()
         self.env.cr.execute("""
-            INSERT INTO mailing_contact_list_rel (contact_id, list_id)
+            INSERT INTO mailing_subscription (contact_id, list_id)
             SELECT st.contact_id AS contact_id, %s AS list_id
             FROM
                 (
@@ -200,7 +240,7 @@ class MassMailingList(models.Model):
                     row_number() OVER (PARTITION BY email ORDER BY email) AS rn
                 FROM
                     mailing_contact contact,
-                    mailing_contact_list_rel contact_list_rel,
+                    mailing_subscription contact_list_rel,
                     mailing_list list
                 WHERE contact.id=contact_list_rel.contact_id
                 AND COALESCE(contact_list_rel.opt_out,FALSE) = FALSE
@@ -212,20 +252,99 @@ class MassMailingList(models.Model):
                     SELECT 1
                     FROM
                         mailing_contact contact2,
-                        mailing_contact_list_rel contact_list_rel2
+                        mailing_subscription contact_list_rel2
                     WHERE contact2.email = contact.email
                     AND contact_list_rel2.contact_id = contact2.id
                     AND contact_list_rel2.list_id = %s
                     )
                 ) st
             WHERE st.rn = 1;""", (self.id, tuple(src_lists.ids), self.id))
-        self.flush()
-        self.invalidate_cache()
+        self.env.invalidate_all()
         if archive:
             (src_lists - self).action_archive()
 
-    def close_dialog(self):
-        return {'type': 'ir.actions.act_window_close'}
+    # ------------------------------------------------------
+    # SUBSCRIPTION MANAGEMENT
+    # ------------------------------------------------------
+
+    def _update_subscription_from_email(self, email, opt_out=True, force_message=None):
+        """ When opting-out: we have to switch opted-in subscriptions. We don't
+        need to create subscription for other lists as opt-out = not being a
+        member.
+
+        When opting-in: we have to switch opted-out subscriptions and create
+        subscription for other mailing lists id they are public. Indeed a
+        contact is opted-in when being subscribed in a mailing list.
+
+        :param str email: email address that should opt-in or opt-out from
+          mailing lists;
+        :param boolean opt_out: if True, opt-out from lists given by self if
+          'email' is member of it. If False, opt-in in lists givben by self
+          and create membership if not already member;
+        :param str force_message: if given, post a note using that body on
+          contact instead of generated update message. Give False to entirely
+          skip the note step;
+        """
+        email_normalized = tools.email_normalize(email)
+        if not self or not email_normalized:
+            return
+
+        contacts = self.env['mailing.contact'].with_context(active_test=False).search(
+            [('email_normalized', '=', email_normalized)]
+        )
+        if not contacts:
+            return
+
+        # switch opted-in subscriptions
+        if opt_out:
+            current_opt_in = contacts.subscription_ids.filtered(
+                lambda sub: not sub.opt_out and sub.list_id in self
+            )
+            if current_opt_in:
+                current_opt_in.write({'opt_out': True})
+        # switch opted-out subscription and create missing subscriptions
+        else:
+            subscriptions = contacts.subscription_ids.filtered(lambda sub: sub.list_id in self)
+            current_opt_out = subscriptions.filtered('opt_out')
+            if current_opt_out:
+                current_opt_out.write({'opt_out': False})
+
+            # create a subscription (for a single contact) for missing lists
+            missing_lists = self - subscriptions.list_id
+            if missing_lists:
+                self.env['mailing.subscription'].create([
+                    {'contact_id': contacts[0].id,
+                     'list_id': mailing_list.id}
+                    for mailing_list in missing_lists
+                ])
+
+        for contact in contacts:
+            # do not log if no opt-out / opt-in was actually done
+            if opt_out:
+                updated = current_opt_in.filtered(lambda sub: sub.contact_id == contact).list_id
+            else:
+                updated = current_opt_out.filtered(lambda sub: sub.contact_id == contact).list_id + missing_lists
+            if not updated:
+                continue
+
+            if force_message is False:
+                continue
+            if force_message:
+                body = force_message
+            elif opt_out:
+                body = Markup('<p>%s</p><ul>%s</ul>') % (
+                    _('%(contact_name)s unsubscribed from the following mailing list(s)', contact_name=contact.display_name),
+                    Markup().join(Markup('<li>%s</li>') % name for name in updated.mapped('name')),
+                )
+            else:
+                body = Markup('<p>%s</p><ul>%s</ul>') % (
+                    _('%(contact_name)s subscribed to the following mailing list(s)', contact_name=contact.display_name),
+                    Markup().join(Markup('<li>%s</li>') % name for name in updated.mapped('name')),
+                )
+            contact.with_context(mail_post_autofollow_author_skip=True).message_post(
+                body=body,
+                subtype_id=self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note'),
+            )
 
     # ------------------------------------------------------
     # MAILING
@@ -270,7 +389,7 @@ class MassMailingList(models.Model):
                 SELECT
                     {','.join(self._get_contact_statistics_fields().values())}
                 FROM
-                    mailing_contact_list_rel r
+                    mailing_subscription r
                     {self._get_contact_statistics_joins()}
                 WHERE list_id IN %s
                 GROUP BY
@@ -288,7 +407,7 @@ class MassMailingList(models.Model):
             if mass_mailing.id not in contact_counts:
                 contact_counts[mass_mailing.id] = {
                     field: 0
-                    for field in self._get_contact_statistics_fields().keys()
+                    for field in mass_mailing._get_contact_statistics_fields()
                 }
 
         return contact_counts
@@ -315,7 +434,7 @@ class MassMailingList(models.Model):
             'contact_count_opt_out': '''
                 SUM(CASE WHEN COALESCE(r.opt_out,FALSE) = TRUE
                     THEN 1 ELSE 0 END) AS contact_count_opt_out''',
-            'contact_count_blacklisted': f'''
+            'contact_count_blacklisted': '''
                 SUM(CASE WHEN bl.id IS NOT NULL
                 THEN 1 ELSE 0 END) AS contact_count_blacklisted'''
         }
