@@ -4,11 +4,11 @@
 from odoo import models, _
 
 
-class Course(models.Model):
+class SlideChannel(models.Model):
     _inherit = "slide.channel"
 
     def action_mass_mailing_attendees(self):
-        domain = [('slide_channel_ids', 'in', self.ids)]
+        domain = repr([('slide_channel_ids', 'in', self.ids)])
         mass_mailing_action = dict(
             name=_('Mass Mail Course Members'),
             type='ir.actions.act_window',
