@@ -4,6 +4,7 @@ from . import mrp_bom
 from . import product
 from . import res_company
 from . import res_partner
+from . import stock_location
 from . import stock_move
 from . import stock_move_line
 from . import stock_picking
@@ -11,3 +12,4 @@ from . import stock_quant
 from . import stock_rule
 from . import stock_warehouse
 from . import mrp_production
+from . import mrp_unbuild
