@@ -1,11 +1,15 @@
 # -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-
+from . import common_setup_methods
 from . import test_point_of_sale_flow
 from . import test_frontend
+from . import test_performances
 from . import test_point_of_sale_ui
 from . import test_anglo_saxon
 from . import test_point_of_sale
+from . import test_pos_controller
+from . import test_pos_invoice_consolidation
+from . import test_pos_cash_rounding
 from . import test_pos_setup
 from . import test_pos_simple_orders
 from . import test_pos_simple_invoiced_orders
@@ -16,4 +20,8 @@ from . import test_pos_multiple_receivable_accounts
 from . import test_pos_other_currency_config
 from . import test_pos_with_fiscal_position
 from . import test_pos_stock_account
-from . import test_js
+from . import test_report_pos_order
+from . import test_report_session
+from . import test_res_config_settings
+from . import test_pos_product_variants
+from . import test_generic_localization
