@@ -2,9 +2,7 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 {
     'name': "Product Matrix",
-    'summary': """
-       Technical module: Matrix Implementation
-    """,
+    'summary': "Technical module: Matrix Implementation",
     'description': """
 Please refer to Sale Matrix or Purchase Matrix for the use of this module.
     """,
@@ -13,6 +11,7 @@ Please refer to Sale Matrix or Purchase Matrix for the use of this module.
     'depends': ['account'],
     # Account dependency for section_and_note widget.
     'data': [
+        'data/res_groups.xml',
         'views/matrix_templates.xml',
     ],
     'demo': [
@@ -20,12 +19,12 @@ Please refer to Sale Matrix or Purchase Matrix for the use of this module.
     ],
     'assets': {
         'web.assets_backend': [
-            'product_matrix/static/src/js/section_and_note_widget.js',
+            'product_matrix/static/src/js/matrix_configurator_hook.js',
+            'product_matrix/static/src/js/product_matrix_dialog.js',
             'product_matrix/static/src/scss/product_matrix.scss',
-        ],
-        'web.assets_qweb': [
             'product_matrix/static/src/xml/**/*',
         ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
