@@ -4,20 +4,17 @@
 {
     'name': 'Purchase Requisition Stock',
     'version': '1.2',
-    'category': 'Operations/Purchase',
+    'category': 'Supply Chain/Purchase',
     'sequence': 70,
-    'summary': '',
-    'description': "",
     'depends': ['purchase_requisition', 'purchase_stock'],
-    'demo': [
-        'data/purchase_requisition_stock_demo.xml'
-        ],
     'data': [
         'security/ir.model.access.csv',
         'data/purchase_requisition_stock_data.xml',
+        'views/purchase_views.xml',
         'views/purchase_requisition_views.xml',
     ],
     'installable': True,
     'auto_install': True,
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
