@@ -4,10 +4,9 @@
 {
     'name': 'Purchase Stock',
     'version': '1.2',
-    'category': 'Inventory/Purchase',
+    'category': 'Supply Chain/Purchase',
     'sequence': 60,
     'summary': 'Purchase Orders, Receipts, Vendor Bills for Stock',
-    'description': "",
     'depends': ['stock_account', 'purchase'],
     'data': [
         'security/ir.model.access.csv',
@@ -16,16 +15,19 @@
         'report/vendor_delay_report.xml',
         'views/purchase_views.xml',
         'views/stock_views.xml',
+        'views/stock_picking_views.xml',
         'views/stock_rule_views.xml',
         'views/res_config_settings_views.xml',
         'views/res_partner_views.xml',
-        'views/stock_production_lot_views.xml',
-        'views/product_category_views.xml',
+        'views/stock_lot_views.xml',
+        'views/stock_orderpoint_views.xml',
+        'views/stock_reference_views.xml',
+        'views/product_views.xml',
         'report/purchase_report_views.xml',
         'report/purchase_report_templates.xml',
-        'report/report_stock_forecasted.xml',
         'report/report_stock_rule.xml',
-        'wizard/stock_replenishment_info.xml'
+        'wizard/stock_replenishment_info.xml',
+        'wizard/product_replenish_views.xml',
     ],
     'demo': [
         'data/purchase_stock_demo.xml',
@@ -35,8 +37,15 @@
     'post_init_hook': '_create_buy_rules',
     'assets': {
         'web.assets_backend': [
-            'purchase_stock/static/**/*',
+            'purchase_stock/static/src/**/*',
+        ],
+        'web.assets_tests': [
+            'purchase_stock/static/tests/tours/**/*',
+        ],
+        'web.assets_unit_tests': [
+            'purchase_stock/static/tests/*.test.js',
         ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
