@@ -15,21 +15,13 @@ This module allow to reinvoice employee expense, by setting the SO directly on t
 """,
     'depends': ['sale_management', 'hr_expense'],
     'data': [
+        'data/sale_expense_data.xml',
         'views/product_view.xml',
         'views/hr_expense_views.xml',
         'views/sale_order_views.xml',
     ],
-    'demo': ['data/sale_expense_demo.xml'],
-    'test': [],
     'installable': True,
     'auto_install': True,
-    'assets': {
-        'web.assets_backend': [
-            'sale_expense/static/src/**/*',
-        ],
-        'web.qunit_suite_tests': [
-            'sale_expense/static/tests/**/*',
-        ],
-    },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
