@@ -1,5 +1,6 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from . import sale_report
 from . import report_stock_rule
+from . import sale_report
+from . import stock_forecasted
+from . import stock_valuation_report
