@@ -1,10 +1,16 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-from odoo.addons.stock_account.tests.test_anglo_saxon_valuation_reconciliation_common import ValuationReconciliationTestCommon
 from datetime import timedelta
+
+from freezegun import freeze_time
+
 from odoo import fields
-from odoo.tests import common, tagged
+from odoo.fields import Command
+from odoo.tests import tagged
+
+from odoo.addons.stock_account.tests.test_anglo_saxon_valuation_reconciliation_common import (
+    ValuationReconciliationTestCommon,
+)
 
 
 @tagged('post_install', '-at_install')
@@ -16,19 +22,19 @@ class TestSaleExpectedDate(ValuationReconciliationTestCommon):
 
         product_A = Product.create({
             'name': 'Product A',
-            'type': 'product',
+            'is_storable': True,
             'sale_delay': 5,
             'uom_id': 1,
         })
         product_B = Product.create({
             'name': 'Product B',
-            'type': 'product',
+            'is_storable': True,
             'sale_delay': 10,
             'uom_id': 1,
         })
         product_C = Product.create({
             'name': 'Product C',
-            'type': 'product',
+            'is_storable': True,
             'sale_delay': 15,
             'uom_id': 1,
         })
@@ -37,13 +43,13 @@ class TestSaleExpectedDate(ValuationReconciliationTestCommon):
         self.env['stock.quant']._update_available_quantity(product_B, self.company_data['default_warehouse'].lot_stock_id, 10)
         self.env['stock.quant']._update_available_quantity(product_C, self.company_data['default_warehouse'].lot_stock_id, 10)
 
-        sale_order = self.env['sale.order'].create({
+        sale_order = self.env['sale.order'].sudo().create({
             'partner_id': self.env['res.partner'].create({'name': 'A Customer'}).id,
             'picking_policy': 'direct',
             'order_line': [
-                (0, 0, {'name': product_A.name, 'product_id': product_A.id, 'customer_lead': product_A.sale_delay, 'product_uom_qty': 5}),
-                (0, 0, {'name': product_B.name, 'product_id': product_B.id, 'customer_lead': product_B.sale_delay, 'product_uom_qty': 5}),
-                (0, 0, {'name': product_C.name, 'product_id': product_C.id, 'customer_lead': product_C.sale_delay, 'product_uom_qty': 5})
+                Command.create({'product_id': product_A.id, 'product_uom_qty': 5}),
+                Command.create({'product_id': product_B.id, 'product_uom_qty': 5}),
+                Command.create({'product_id': product_C.id, 'product_uom_qty': 5})
             ],
         })
 
@@ -82,8 +88,7 @@ class TestSaleExpectedDate(ValuationReconciliationTestCommon):
 
         # Check effective date, it should be date on which the first shipment successfully delivered to customer
         picking = sale_order.picking_ids[0]
-        for ml in picking.move_line_ids:
-            ml.qty_done = ml.product_uom_qty
+        picking.move_ids.picked = True
         picking._action_done()
         self.assertEqual(picking.state, 'done', "Picking not processed correctly!")
         self.assertEqual(fields.Date.today(), sale_order.effective_date.date(), "Wrong effective date on sale order!")
@@ -92,17 +97,17 @@ class TestSaleExpectedDate(ValuationReconciliationTestCommon):
 
         # In order to test the Commitment Date feature in Sales Orders in Odoo,
         # I copy a demo Sales Order with committed Date on 2010-07-12
-        new_order = self.env['sale.order'].create({
+        new_order = self.env['sale.order'].sudo().create({
             'partner_id': self.env['res.partner'].create({'name': 'A Partner'}).id,
-            'order_line': [(0, 0, {
-                'name': "A product",
-                'product_id': self.env['product.product'].create({
-                    'name': 'A product',
-                    'type': 'product',
-                }).id,
-                'product_uom_qty': 1,
-                'price_unit': 750,
-            })],
+            'order_line': [
+                Command.create({
+                    'product_id': self.env['product.product'].create({
+                        'name': 'A product',
+                        'is_storable': True,
+                    }).id,
+                    'price_unit': 750,
+                })
+            ],
             'commitment_date': '2010-07-12',
         })
         # I confirm the Sales Order.
@@ -113,3 +118,88 @@ class TestSaleExpectedDate(ValuationReconciliationTestCommon):
         right_date = commitment_date - security_delay
         for line in new_order.order_line:
             self.assertEqual(line.move_ids[0].date, right_date, "The expected date for the Stock Move is wrong")
+
+    def test_expected_date_with_storable_product(self):
+        ''' This test ensures the expected date is computed based on only goods(consu) products.
+        It's avoiding computation for non-goods products.
+        '''
+        sale_delay = 10.0
+        self.product.sale_delay = sale_delay
+
+        # Create a sale order with a consu product.
+        sale_order = self.env['sale.order'].sudo().create({
+            'partner_id': self.partner.id,
+            'order_line': [Command.create({
+                'product_id': self.product.id,
+                'product_uom_qty': 1000,
+            })],
+        })
+
+        # Ensure that expected date is correctly computed based on the consu product's sale delay.
+        self.assertEqual(sale_order.expected_date, fields.Datetime.now() + timedelta(days=sale_delay))
+
+        # Add a service product and ensure the expected date remains unchanged.
+        sale_order.write({
+            'order_line': [Command.create({
+                'product_id': self.service_product.id,
+                'product_uom_qty': 1000,
+            })],
+        })
+        self.assertEqual(sale_order.expected_date, fields.Datetime.now() + timedelta(days=sale_delay))
+
+    def test_invoice_delivery_date(self):
+        self.env['stock.quant']._update_available_quantity(
+            self.test_product_order,
+            self.company_data['default_warehouse'].lot_stock_id,
+            75.0,
+        )
+        order = self.env['sale.order'].sudo().create({
+            'partner_id': self.partner_a.id,
+            'picking_policy': 'one',
+            'order_line': [Command.create({
+                'product_id': self.test_product_order.id,
+                'product_uom_qty': 100.0,
+            })],
+        })
+        order.action_confirm()
+        picking_1 = order.picking_ids
+        picking_1.move_ids.picked = True
+        invoice = order._create_invoices()
+        self.assertFalse(invoice.delivery_date)
+        picking_1._action_done()
+        self.assertTrue(order.effective_date, "Effective date should exist after done picking")
+        effective_date = order.effective_date.date()
+        self.assertEqual(
+            invoice.delivery_date, effective_date,
+            "Default invoice delivery date should equal effective date",
+        )
+
+        self.env['stock.quant']._update_available_quantity(
+            self.test_product_order,
+            self.company_data['default_warehouse'].lot_stock_id,
+            25.0,
+        )
+        with freeze_time(effective_date + timedelta(days=3)):
+            custom_delivery_date = fields.Date.today()
+            picking_2 = (order.picking_ids - picking_1).ensure_one()
+            picking_2.move_ids.write({'quantity': 25.0, 'picked': True})
+            picking_2._action_done()
+            self.assertEqual(
+                invoice.delivery_date, effective_date,
+                "Invoice delivery date should default to earliest picking date",
+            )
+            product_line = invoice.line_ids[0]
+            invoice.write({
+                'delivery_date': custom_delivery_date,
+                'line_ids': [Command.update(product_line.id, {'quantity': 0.0})],
+            })
+            product_line.quantity += 75.0
+            self.assertEqual(
+                invoice.delivery_date, custom_delivery_date,
+                "Custom invoice delivery shouldn't change after line change",
+            )
+            invoice.action_post()
+            self.assertEqual(
+                invoice.delivery_date, custom_delivery_date,
+                "Custom invoice delivery shouldn't change posting invoice",
+            )
