@@ -2,25 +2,23 @@
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from ast import literal_eval
+from uuid import uuid4
 
 from odoo import api, fields, models, _
-from odoo.addons.phone_validation.tools import phone_validation
+from odoo.addons.sms.tools.sms_tools import sms_content_to_rendered_html
 from odoo.exceptions import UserError
-from odoo.tools import html2plaintext
 
 
-class SendSMS(models.TransientModel):
+class SmsComposer(models.TransientModel):
     _name = 'sms.composer'
     _description = 'Send SMS Wizard'
 
     @api.model
     def default_get(self, fields):
-        result = super(SendSMS, self).default_get(fields)
+        result = super().default_get(fields)
 
         result['res_model'] = result.get('res_model') or self.env.context.get('active_model')
 
-        if not result.get('active_domain'):
-            result['active_domain'] = repr(self.env.context.get('active_domain', []))
         if not result.get('res_ids'):
             if not result.get('res_id') and self.env.context.get('active_ids') and len(self.env.context.get('active_ids')) > 1:
                 result['res_ids'] = repr(self.env.context.get('active_ids'))
@@ -35,34 +33,32 @@ class SendSMS(models.TransientModel):
         ('numbers', 'Send to numbers'),
         ('comment', 'Post on a document'),
         ('mass', 'Send SMS in batch')], string='Composition Mode',
-        compute='_compute_composition_mode', readonly=False, required=True, store=True)
+        compute='_compute_composition_mode', precompute=True, readonly=False, required=True, store=True)
     res_model = fields.Char('Document Model Name')
+    res_model_description = fields.Char('Document Model Description', compute='_compute_res_model_description')
     res_id = fields.Integer('Document ID')
     res_ids = fields.Char('Document IDs')
     res_ids_count = fields.Integer(
-        'Visible records count', compute='_compute_recipients_count', compute_sudo=False,
+        'Visible records count', compute='_compute_res_ids_count', compute_sudo=False,
         help='Number of recipients that will receive the SMS if sent in mass mode, without applying the Active Domain value')
-    use_active_domain = fields.Boolean('Use active domain')
-    active_domain = fields.Text('Active domain', readonly=True)
-    active_domain_count = fields.Integer(
-        'Active records count', compute='_compute_recipients_count', compute_sudo=False,
-        help='Number of records found when searching with the value in Active Domain')
     comment_single_recipient = fields.Boolean(
         'Single Mode', compute='_compute_comment_single_recipient', compute_sudo=False,
         help='Indicates if the SMS composer targets a single specific recipient')
     # options for comment and mass mode
     mass_keep_log = fields.Boolean('Keep a note on document', default=True)
     mass_force_send = fields.Boolean('Send directly', default=False)
-    mass_use_blacklist = fields.Boolean('Use blacklist', default=True)
+    use_exclusion_list = fields.Boolean(
+        'Use Exclusion List', default=True, copy=False,
+        help='Prevent sending messages to blacklisted contacts. Disable only when absolutely necessary.')
     # recipients
     recipient_valid_count = fields.Integer('# Valid recipients', compute='_compute_recipients', compute_sudo=False)
     recipient_invalid_count = fields.Integer('# Invalid recipients', compute='_compute_recipients', compute_sudo=False)
-    recipient_single_description = fields.Text('Recipients (Partners)', compute='_compute_recipient_single', compute_sudo=False)
-    recipient_single_number = fields.Char('Stored Recipient Number', compute='_compute_recipient_single', compute_sudo=False)
+    recipient_single_description = fields.Text('Recipients (Partners)', compute='_compute_recipient_single_non_stored', compute_sudo=False)
+    recipient_single_number = fields.Char('Stored Recipient Number', compute='_compute_recipient_single_non_stored', compute_sudo=False)
     recipient_single_number_itf = fields.Char(
-        'Recipient Number', compute='_compute_recipient_single',
+        'Recipient Number', compute='_compute_recipient_single_stored',
         readonly=False, compute_sudo=False, store=True,
-        help='UX field allowing to edit the recipient number. If changed it will be stored onto the recipient.')
+        help='Phone number of the recipient. If changed, it will be recorded on recipient\'s profile.')
     recipient_single_valid = fields.Boolean("Is valid", compute='_compute_recipient_single_valid', compute_sudo=False)
     number_field_name = fields.Char('Number Field')
     numbers = fields.Char('Recipients (Numbers)')
@@ -71,33 +67,35 @@ class SendSMS(models.TransientModel):
     template_id = fields.Many2one('sms.template', string='Use Template', domain="[('model', '=', res_model)]")
     body = fields.Text(
         'Message', compute='_compute_body',
-        readonly=False, store=True, required=True)
+        precompute=True, readonly=False, store=True, required=True)
 
-    @api.depends('res_ids_count', 'active_domain_count')
+    @api.depends('res_ids_count')
     @api.depends_context('sms_composition_mode')
     def _compute_composition_mode(self):
         for composer in self:
             if self.env.context.get('sms_composition_mode') == 'guess' or not composer.composition_mode:
-                if composer.res_ids_count > 1 or (composer.use_active_domain and composer.active_domain_count > 1):
+                if composer.res_ids_count > 1:
                     composer.composition_mode = 'mass'
                 else:
                     composer.composition_mode = 'comment'
 
-    @api.depends('res_model', 'res_id', 'res_ids', 'active_domain')
-    def _compute_recipients_count(self):
+    @api.depends('res_model')
+    def _compute_res_model_description(self):
+        self.res_model_description = False
+        for composer in self.filtered('res_model'):
+            composer.res_model_description = self.env['ir.model']._get(composer.res_model).display_name
+
+    @api.depends('res_model', 'res_id', 'res_ids')
+    def _compute_res_ids_count(self):
         for composer in self:
             composer.res_ids_count = len(literal_eval(composer.res_ids)) if composer.res_ids else 0
-            if composer.res_model:
-                composer.active_domain_count = self.env[composer.res_model].search_count(literal_eval(composer.active_domain or '[]'))
-            else:
-                composer.active_domain_count = 0
 
     @api.depends('res_id', 'composition_mode')
     def _compute_comment_single_recipient(self):
         for composer in self:
             composer.comment_single_recipient = bool(composer.res_id and composer.composition_mode == 'comment')
 
-    @api.depends('res_model', 'res_id', 'res_ids', 'use_active_domain', 'composition_mode', 'number_field_name', 'sanitized_numbers')
+    @api.depends('res_model', 'res_id', 'res_ids', 'composition_mode', 'number_field_name', 'sanitized_numbers')
     def _compute_recipients(self):
         for composer in self:
             composer.recipient_valid_count = 0
@@ -107,41 +105,50 @@ class SendSMS(models.TransientModel):
                 continue
 
             records = composer._get_records()
-            if records and issubclass(type(records), self.pool['mail.thread']):
+            if records:
                 res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=not composer.comment_single_recipient)
                 composer.recipient_valid_count = len([rid for rid, rvalues in res.items() if rvalues['sanitized']])
                 composer.recipient_invalid_count = len([rid for rid, rvalues in res.items() if not rvalues['sanitized']])
             else:
                 composer.recipient_invalid_count = 0 if (
-                    composer.sanitized_numbers or (composer.composition_mode == 'mass' and composer.use_active_domain)
+                    composer.sanitized_numbers or composer.composition_mode == 'mass'
                 ) else 1
 
     @api.depends('res_model', 'number_field_name')
-    def _compute_recipient_single(self):
+    def _compute_recipient_single_stored(self):
         for composer in self:
             records = composer._get_records()
-            if not records or not issubclass(type(records), self.pool['mail.thread']) or not composer.comment_single_recipient:
-                composer.recipient_single_description = False
-                composer.recipient_single_number = ''
+            if not records or not composer.comment_single_recipient:
                 composer.recipient_single_number_itf = ''
                 continue
             records.ensure_one()
-            res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=False)
-            composer.recipient_single_description = res[records.id]['partner'].name or records.display_name
-            composer.recipient_single_number = res[records.id]['number'] or ''
+            # If the composer was opened with a specific field use that, otherwise get the partner's
+            res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=not composer.number_field_name)
             if not composer.recipient_single_number_itf:
-                composer.recipient_single_number_itf = res[records.id]['number'] or ''
+                composer.recipient_single_number_itf = res[records.id]['sanitized'] or res[records.id]['number'] or ''
             if not composer.number_field_name:
                 composer.number_field_name = res[records.id]['field_store']
 
+    @api.depends('res_model', 'number_field_name')
+    def _compute_recipient_single_non_stored(self):
+        for composer in self:
+            records = composer._get_records()
+            if not records or not composer.comment_single_recipient:
+                composer.recipient_single_description = False
+                composer.recipient_single_number = ''
+                continue
+            records.ensure_one()
+            res = records._sms_get_recipients_info(force_field=composer.number_field_name, partner_fallback=True)
+            composer.recipient_single_description = res[records.id]['partner'].name or records._mail_get_partners()[records[0].id].display_name
+            composer.recipient_single_number = res[records.id]['sanitized'] or res[records.id]['number'] or ''
+
     @api.depends('recipient_single_number', 'recipient_single_number_itf')
     def _compute_recipient_single_valid(self):
         for composer in self:
             value = composer.recipient_single_number_itf or composer.recipient_single_number
             if value:
                 records = composer._get_records()
-                sanitized = phone_validation.phone_sanitize_numbers_w_record([value], records)[value]['sanitized']
-                composer.recipient_single_valid = bool(sanitized)
+                composer.recipient_single_valid = bool(records._phone_format(number=value)) if len(records) == 1 else False
             else:
                 composer.recipient_single_valid = False
 
@@ -151,9 +158,8 @@ class SendSMS(models.TransientModel):
             if composer.numbers:
                 record = composer._get_records() if composer.res_model and composer.res_id else self.env.user
                 numbers = [number.strip() for number in composer.numbers.split(',')]
-                sanitize_res = phone_validation.phone_sanitize_numbers_w_record(numbers, record)
-                sanitized_numbers = [info['sanitized'] for info in sanitize_res.values() if info['sanitized']]
-                invalid_numbers = [number for number, info in sanitize_res.items() if info['code']]
+                sanitized_numbers = [record._phone_format(number=number) for number in numbers]
+                invalid_numbers = [number for sanitized, number in zip(sanitized_numbers, numbers) if not sanitized]
                 if invalid_numbers:
                     raise UserError(_('Following numbers are not correctly encoded: %s', repr(invalid_numbers)))
                 composer.sanitized_numbers = ','.join(sanitized_numbers)
@@ -164,26 +170,11 @@ class SendSMS(models.TransientModel):
     def _compute_body(self):
         for record in self:
             if record.template_id and record.composition_mode == 'comment' and record.res_id:
-                record.body = record.template_id._render_field('body', [record.res_id], compute_lang=True)[record.res_id]
+                additional_context = record._get_additional_render_context().get('body', {})
+                record.body = record.template_id._render_field('body', [record.res_id], compute_lang=True, add_context=additional_context)[record.res_id]
             elif record.template_id:
                 record.body = record.template_id.body
 
-    # ------------------------------------------------------------
-    # CRUD
-    # ------------------------------------------------------------
-
-    @api.model
-    def create(self, values):
-        # TDE FIXME: currently have to compute manually to avoid required issue, waiting VFE branch
-        if not values.get('body') or not values.get('composition_mode'):
-            values_wdef = self._add_missing_default_values(values)
-            cache_composer = self.new(values_wdef)
-            cache_composer._compute_body()
-            cache_composer._compute_composition_mode()
-            values['body'] = values.get('body') or cache_composer.body
-            values['composition_mode'] = values.get('composition_mode') or cache_composer.composition_mode
-        return super(SendSMS, self).create(values)
-
     # ------------------------------------------------------------
     # Actions
     # ------------------------------------------------------------
@@ -207,7 +198,7 @@ class SendSMS(models.TransientModel):
         if self.composition_mode == 'numbers':
             return self._action_send_sms_numbers()
         elif self.composition_mode == 'comment':
-            if records is None or not issubclass(type(records), self.pool['mail.thread']):
+            if records is None or not isinstance(records, self.pool['mail.thread']):
                 return self._action_send_sms_numbers()
             if self.comment_single_recipient:
                 return self._action_send_sms_comment_single(records)
@@ -217,12 +208,17 @@ class SendSMS(models.TransientModel):
             return self._action_send_sms_mass(records)
 
     def _action_send_sms_numbers(self):
-        self.env['sms.api']._send_sms_batch([{
-            'res_id': 0,
-            'number': number,
-            'content': self.body,
-        } for number in self.sanitized_numbers.split(',')])
-        return True
+        sms_values = [
+            {
+                'body': self.body,
+                'number': number
+            } for number in (
+                self.sanitized_numbers.split(',') if self.sanitized_numbers else [self.recipient_single_number_itf or self.recipient_single_number or '']
+            )
+        ]
+        sms_su = self.env['sms.sms'].sudo().create(sms_values)
+        sms_su.send()
+        return sms_su
 
     def _action_send_sms_comment_single(self, records=None):
         # If we have a recipient_single_original number, it's possible this number has been corrected in the popup
@@ -230,7 +226,9 @@ class SendSMS(models.TransientModel):
         # on the numbers in the database.
         records = records if records is not None else self._get_records()
         records.ensure_one()
-        if self.recipient_single_number_itf and self.recipient_single_number_itf != self.recipient_single_number:
+        if not self.number_field_name or self.number_field_name not in records:
+            self.numbers = self.recipient_single_number_itf or self.recipient_single_number
+        elif self.recipient_single_number_itf and self.recipient_single_number_itf != self.recipient_single_number:
             records.write({self.number_field_name: self.recipient_single_number_itf})
         return self._action_send_sms_comment(records=records)
 
@@ -239,9 +237,12 @@ class SendSMS(models.TransientModel):
         subtype_id = self.env['ir.model.data']._xmlid_to_res_id('mail.mt_note')
 
         messages = self.env['mail.message']
+        all_bodies = self._prepare_body_values(records)
+
         for record in records:
-            messages |= record._message_sms(
-                self.body, subtype_id=subtype_id,
+            messages += record._message_sms(
+                all_bodies[record.id],
+                subtype_id=subtype_id,
                 number_field=self.number_field_name,
                 sms_numbers=self.sanitized_numbers.split(',') if self.sanitized_numbers else None)
         return messages
@@ -249,15 +250,17 @@ class SendSMS(models.TransientModel):
     def _action_send_sms_mass(self, records=None):
         records = records if records is not None else self._get_records()
 
-        sms_record_values = self._prepare_mass_sms_values(records)
-        sms_all = self._prepare_mass_sms(records, sms_record_values)
-
-        if sms_all and self.mass_keep_log and records and issubclass(type(records), self.pool['mail.thread']):
-            log_values = self._prepare_mass_log_values(records, sms_record_values)
-            records._message_log_batch(**log_values)
+        sms_record_values_filtered = self._filter_out_and_handle_revoked_sms_values(self._prepare_mass_sms_values(records))
+        records_filtered = records.filtered(lambda record: record.id in sms_record_values_filtered)
+        if self.mass_keep_log and sms_record_values_filtered and isinstance(records_filtered, self.pool['mail.thread']):
+            log_values = self._prepare_mass_log_values(records_filtered, sms_record_values_filtered)
+            mail_messages = records_filtered._message_log_batch(**log_values)
+            for idx, record in enumerate(records_filtered):
+                sms_record_values_filtered[record.id]['mail_message_id'] = mail_messages[idx].id
+        sms_all = self._prepare_mass_sms(records_filtered, sms_record_values_filtered)
 
         if sms_all and self.mass_force_send:
-            sms_all.filtered(lambda sms: sms.state == 'outgoing').send(auto_commit=False, raise_exception=False)
+            sms_all.filtered(lambda sms: sms.state == 'outgoing').send(raise_exception=False)
             return self.env['sms.sms'].sudo().search([('id', 'in', sms_all.ids)])
         return sms_all
 
@@ -265,10 +268,19 @@ class SendSMS(models.TransientModel):
     # Mass mode specific
     # ------------------------------------------------------------
 
+    def _filter_out_and_handle_revoked_sms_values(self, sms_values_all):
+        """Meant to be overridden to filter out and handle sms that must not be sent.
+
+        :param dict sms_values_all: sms values by res_id
+        :returns: filtered sms_vals_all
+        :rtype: dict
+        """
+        return sms_values_all
+
     def _get_blacklist_record_ids(self, records, recipients_info):
         """ Get a list of blacklisted records. Those will be directly canceled
         with the right error code. """
-        if self.mass_use_blacklist:
+        if self.use_exclusion_list:
             bl_numbers = self.env['phone.blacklist'].sudo().search([]).mapped('number')
             return [r.id for r in records if recipients_info[r.id]['sanitized'] in bl_numbers]
         return []
@@ -295,10 +307,11 @@ class SendSMS(models.TransientModel):
         return recipients_info
 
     def _prepare_body_values(self, records):
+        additional_context = self._get_additional_render_context().get('body', {})
         if self.template_id and self.body == self.template_id.body:
-            all_bodies = self.template_id._render_field('body', records.ids, compute_lang=True)
+            all_bodies = self.template_id._render_field('body', records.ids, compute_lang=True, add_context=additional_context)
         else:
-            all_bodies = self.env['mail.render.mixin']._render_template(self.body, records._name, records.ids)
+            all_bodies = self.env['mail.render.mixin']._render_template(self.body, records._name, records.ids, add_context=additional_context)
         return all_bodies
 
     def _prepare_mass_sms_values(self, records):
@@ -330,10 +343,11 @@ class SendSMS(models.TransientModel):
 
             result[record.id] = {
                 'body': all_bodies[record.id],
-                'partner_id': recipients['partner'].id,
-                'number': sanitized if sanitized else recipients['number'],
-                'state': state,
                 'failure_type': failure_type,
+                'number': sanitized if sanitized else recipients['number'],
+                'partner_id': recipients['partner'].id,
+                'state': state,
+                'uuid': uuid4().hex,
             }
         return result
 
@@ -344,7 +358,7 @@ class SendSMS(models.TransientModel):
     def _prepare_log_body_values(self, sms_records_values):
         result = {}
         for record_id, sms_values in sms_records_values.items():
-            result[record_id] = html2plaintext(sms_values['body'])
+            result[record_id] = sms_content_to_rendered_html(sms_values['body'])
         return result
 
     def _prepare_mass_log_values(self, records, sms_records_values):
@@ -353,6 +367,18 @@ class SendSMS(models.TransientModel):
             'message_type': 'sms',
         }
 
+    # ------------------------------------------------------------
+    # Render
+    # ------------------------------------------------------------
+
+    def _get_additional_render_context(self):
+        """
+        Return a dict associating fields with their relevant render context if any.
+
+        e.g. {'body': {'additional_value': self.env.context.get('additional_value')}}
+        """
+        return {}
+
     # ------------------------------------------------------------
     # Tools
     # ------------------------------------------------------------
@@ -362,7 +388,8 @@ class SendSMS(models.TransientModel):
         if composition_mode == 'comment':
             if not body and template_id and res_id:
                 template = self.env['sms.template'].browse(template_id)
-                result['body'] = template._render_template(template.body, res_model, [res_id])[res_id]
+                additional_context = self._get_additional_render_context().get('body', {})
+                result['body'] = template._render_template(template.body, res_model, [res_id], add_context=additional_context)[res_id]
             elif template_id:
                 template = self.env['sms.template'].browse(template_id)
                 result['body'] = template.body
@@ -375,10 +402,7 @@ class SendSMS(models.TransientModel):
     def _get_records(self):
         if not self.res_model:
             return None
-        if self.use_active_domain:
-            active_domain = literal_eval(self.active_domain or '[]')
-            records = self.env[self.res_model].search(active_domain)
-        elif self.res_ids:
+        if self.res_ids:
             records = self.env[self.res_model].browse(literal_eval(self.res_ids))
         elif self.res_id:
             records = self.env[self.res_model].browse(self.res_id)
