@@ -9,9 +9,10 @@ from odoo.exceptions import UserError
 class ProductTemplate(models.Model):
     _inherit = "product.template"
 
-    landed_cost_ok = fields.Boolean('Is a Landed Cost', help='Indicates whether the product is a landed cost.')
-    split_method_landed_cost = fields.Selection(SPLIT_METHOD, string="Default Split Method",
-                                                help="Default Split Method when used for Landed Cost")
+    landed_cost_ok = fields.Boolean('Is a Landed Cost', help='Indicates whether the product is a landed cost: when receiving a vendor bill, you can allocate this cost on preceding receipts.')
+    split_method_landed_cost = fields.Selection(
+        SPLIT_METHOD, string="Default Split Method",
+        help="Default Split Method when used for Landed Cost")
 
     def write(self, vals):
         for product in self:
