@@ -3,5 +3,3 @@
 
 from . import stock_picking_to_batch
 from . import stock_add_to_wave
-from . import stock_package_destination
-from . import stock_backorder_confirmation
