@@ -3,19 +3,23 @@
 {
     'name': 'Mass Mail Tests',
     'version': '1.0',
-    'category': 'Hidden',
+    'category': 'Marketing/Email Marketing',
     'sequence': 8765,
     'summary': 'Mass Mail Tests: feature and performance tests for mass mailing',
     'description': """This module contains tests related to mass mailing. Those
-are contained in a separate module to use specific test models defined in
+are present in a separate module to use specific test models defined in
 test_mail. """,
-    'depends': ['test_mail', 'mass_mailing'],
+    'depends': [
+        'mass_mailing',
+        'mass_mailing_sms',
+        'sms_twilio',
+        'test_mail',
+        'test_mail_sms',
+    ],
     'data': [
         'security/ir.model.access.csv',
     ],
-    'demo': [
-    ],
     'installable': True,
-    'application': False,
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
