@@ -1,16 +1,17 @@
 # -*- coding: utf-8 -*-
 {
     'name': "Test testing utilities",
-    'category': "Tests",
+    'category': "Hidden/Tests",
     'description': """Non-trivial testing utilities can require models & all
     
 This here module is useful to validate that they're doing what they're 
 supposed to do
     """,
-    'depends': ['base'],
+    'depends': ['base', 'web'],
     'data': [
-        'ir.model.access.csv',
-        'menu.xml',
+        'security/ir.model.access.csv',
+        'views/menu.xml',
     ],
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
