@@ -17,20 +17,34 @@ models which only purpose is to run tests.""",
         'website',
         'theme_default',
     ],
+    'demo': [
+        'data/test_website_demo.xml',
+    ],
     'data': [
-        'views/templates.xml',
-        'data/test_website_data.xml',
+        'security/test_website_security.xml',
         'security/ir.model.access.csv',
+        'views/templates.xml',
+        'views/test_model_multi_website_views.xml',
+        'views/test_model_views.xml',
+        'data/test_website_data.xml',
     ],
     'installable': True,
-    'application': False,
     'assets': {
+        'test_website.test_bundle': [
+            'http://test.external.link/javascript1.js',
+            '/web/static/src/libs/fontawesome/css/font-awesome.css',
+            'http://test.external.link/style1.css',
+            '/web/static/src/module_loader.js',
+            'http://test.external.link/javascript2.js',
+            'http://test.external.link/style2.css',
+        ],
         'web.assets_frontend': [
-            'test_website/static/src/js/test_error.js',
+            'test_website/static/src/interactions/**/*',
         ],
         'web.assets_tests': [
-            'test_website/static/tests/**/*',
+            'test_website/static/tests/tours/*',
         ],
     },
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
