@@ -1,5 +1,3 @@
-/** @odoo-module **/
-
 import {
     serializeDate,
     serializeDateTime,
@@ -9,46 +7,73 @@ import {
 import { localization } from "@web/core/l10n/localization";
 import { _t } from "@web/core/l10n/translation";
 import { registry } from "@web/core/registry";
+import { user } from "@web/core/user";
 import { KeepLast } from "@web/core/utils/concurrency";
-import { Model } from "@web/views/model";
+import { Model } from "@web/model/model";
+import { extractFieldsFromArchInfo } from "@web/model/relational_model/utils";
+import { browser } from "@web/core/browser/browser";
+import { makeContext } from "@web/core/context";
+import { groupBy, intersection } from "@web/core/utils/arrays";
+import { Cache } from "@web/core/utils/cache";
+import { formatFloat } from "@web/core/utils/numbers";
+import { useDebounced } from "@web/core/utils/timing";
+import { computeAggregatedValue } from "@web/views/utils";
+
+const { DateTime } = luxon;
 
 export class CalendarModel extends Model {
-    setup(params, services) {
-        /** @protected */
-        this.user = services.user;
+    static DEBOUNCED_LOAD_DELAY = 600;
+    static services = ["notification"];
 
+    setup(params, { notification }) {
         /** @protected */
         this.keepLast = new KeepLast();
+        this.notification = notification;
 
         const formViewFromConfig = (this.env.config.views || []).find((view) => view[1] === "form");
         const formViewIdFromConfig = formViewFromConfig ? formViewFromConfig[0] : false;
+        const fieldNodes = params.popoverFieldNodes;
+        const { activeFields, fields } = extractFieldsFromArchInfo({ fieldNodes }, params.fields);
         this.meta = {
             ...params,
-            firstDayOfWeek: localization.weekStart,
+            activeFields,
+            fields,
+            firstDayOfWeek: (localization.weekStart || 0) % 7,
             formViewId: params.formViewId || formViewIdFromConfig,
         };
-
+        if (this.meta.aggregate?.split(":").length === 1) {
+            const aggregator = this.fields[this.meta.aggregate].aggregator || "sum";
+            this.meta.aggregate = `${this.meta.aggregate}:${aggregator}`;
+        }
+        this.meta.scale = this.getLocalStorageScale();
         this.data = {
-            filters: {},
             filterSections: {},
-            hasCreateRight: null,
             range: null,
             records: {},
             unusualDays: [],
         };
+
+        const debouncedLoadDelay = this.constructor.DEBOUNCED_LOAD_DELAY;
+        this.debouncedLoad = useDebounced((params) => this.load(params), debouncedLoadDelay);
+
+        this._unusualDaysCache = new Cache(
+            (data) => this.fetchUnusualDays(data),
+            (data) => `${serializeDateTime(data.range.start)},${serializeDateTime(data.range.end)}`
+        );
     }
     async load(params = {}) {
         Object.assign(this.meta, params);
         if (!this.meta.date) {
             this.meta.date =
                 params.context && params.context.initial_date
-                    ? deserializeDateTime(params.context.initial_date)
-                    : luxon.DateTime.local();
+                    ? deserializeDateTime(params.context.initial_date).startOf("day")
+                    : DateTime.local().startOf("day");
         }
         // Prevent picking a scale that is not supported by the view
         if (!this.meta.scales.includes(this.meta.scale)) {
             this.meta.scale = this.meta.scales[0];
         }
+        browser.localStorage.setItem(this.storageKey, this.meta.scale);
         const data = { ...this.data };
         await this.keepLast.add(this.updateData(data));
         this.data = data;
@@ -59,23 +84,35 @@ export class CalendarModel extends Model {
     // Public
     //--------------------------------------------------------------------------
 
+    get aggregate() {
+        return this.meta.aggregate;
+    }
     get date() {
         return this.meta.date;
     }
     get canCreate() {
-        return this.meta.canCreate && this.data.hasCreateRight;
+        return this.meta.canCreate;
     }
     get canDelete() {
         return this.meta.canDelete;
     }
     get canEdit() {
-        return !this.meta.fields[this.meta.fieldMapping.date_start].readonly;
+        return this.meta.canEdit && !this.meta.fields[this.meta.fieldMapping.date_start].readonly;
+    }
+    get dateStartType() {
+        return this.fields[this.fieldMapping.date_start].type;
+    }
+    get dateStopType() {
+        if (this.fieldMapping.date_stop) {
+            return this.fields[this.fieldMapping.date_stop].type;
+        }
+        return null;
     }
     get eventLimit() {
         return this.meta.eventLimit;
     }
     get exportedState() {
-        return this.meta;
+        return { date: this.meta.date };
     }
     get fieldMapping() {
         return this.meta.fieldMapping;
@@ -101,8 +138,11 @@ export class CalendarModel extends Model {
     get hasEditDialog() {
         return this.meta.hasEditDialog;
     }
+    get hasMultiCreate() {
+        return !!this.meta.multiCreateView && !this.env.isSmall && this.meta.scale === "month";
+    }
     get hasQuickCreate() {
-        return this.meta.hasQuickCreate;
+        return this.meta.quickCreate;
     }
     get isDateHidden() {
         return this.meta.isDateHidden;
@@ -110,8 +150,14 @@ export class CalendarModel extends Model {
     get isTimeHidden() {
         return this.meta.isTimeHidden;
     }
-    get popoverFields() {
-        return this.meta.popoverFields;
+    get monthOverflow() {
+        return this.meta.monthOverflow;
+    }
+    get popoverFieldNodes() {
+        return this.meta.popoverFieldNodes;
+    }
+    get activeFields() {
+        return this.meta.activeFields;
     }
     get rangeEnd() {
         return this.data.range.end;
@@ -131,25 +177,47 @@ export class CalendarModel extends Model {
     get scales() {
         return this.meta.scales;
     }
+    get showDatePicker() {
+        return this.meta.showDatePicker;
+    }
+    get showMultiCreateTimeRange() {
+        return this.dateStartType === "datetime" && this.dateStopType === "datetime";
+    }
+    get storageKey() {
+        return `scaleOf-viewId-${this.env.config.viewId}`;
+    }
     get unusualDays() {
         return this.data.unusualDays;
     }
+    get quickCreateFormViewId() {
+        return this.meta.quickCreateViewId;
+    }
+    get defaultFilterLabel() {
+        return _t("Undefined");
+    }
 
     //--------------------------------------------------------------------------
 
     async createFilter(fieldName, filterValue) {
         const info = this.meta.filtersInfo[fieldName];
-        if (info && info.writeFieldName && info.writeResModel) {
+        if (!info || !info.writeFieldName || !info.writeResModel) {
+            return;
+        }
+
+        const normalizedFilterValue = Array.isArray(filterValue) ? filterValue : [filterValue];
+        const dataArray = normalizedFilterValue.map((value) => {
             const data = {
-                user_id: this.user.userId,
-                [info.writeFieldName]: filterValue,
+                user_id: user.userId,
+                [info.writeFieldName]: value,
             };
             if (info.filterFieldName) {
                 data[info.filterFieldName] = true;
             }
-            await this.orm.create(info.writeResModel, [data]);
-            await this.load();
-        }
+            return data;
+        });
+
+        await this.orm.create(info.writeResModel, dataArray);
+        await this.load();
     }
     async createRecord(record) {
         const rawRecord = this.buildRawRecord(record);
@@ -157,53 +225,120 @@ export class CalendarModel extends Model {
         await this.orm.create(this.meta.resModel, [rawRecord], { context });
         await this.load();
     }
+
+    /**
+     * Create multi records of the specify dates and values.
+     * Optionally time range can be specified to set the start and end time.
+     * Also, if there is a filter section, the first filter section will be chosen as additional value for the record.
+     *
+     * @param {Object} multiCreateData
+     * @param {DateTime[]} dates array of Date
+     * @returns {Promise<*>}
+     */
+    async multiCreateRecords(multiCreateData, dates) {
+        const records = [];
+        const values = await multiCreateData.record.getChanges();
+        const timeRange = multiCreateData.timeRange;
+
+        // we deliberately only use the values of the first filter section, to avoid combinatorial explosion
+        const [section] = this.filterSections;
+        for (const date of dates) {
+            const initialRecordValue = {};
+            if (this.showMultiCreateTimeRange) {
+                initialRecordValue.start = date.plus(timeRange.start.toObject());
+                initialRecordValue.end = date.plus(timeRange.end.toObject());
+            } else {
+                initialRecordValue.start = date;
+            }
+            const rawRecord = this.buildRawRecord(initialRecordValue);
+            if (!section) {
+                records.push({
+                    ...rawRecord,
+                    ...values,
+                });
+                continue;
+            }
+            for (const filter of section.filters) {
+                if (filter.active && filter.type === "record") {
+                    records.push({
+                        ...rawRecord,
+                        ...values,
+                        [section.fieldName]: filter.value,
+                    });
+                }
+            }
+        }
+        if (records.length) {
+            const createdRecords = await this.orm.create(this.meta.resModel, records, {
+                context: this.meta.context,
+            });
+            await this.load();
+            return createdRecords;
+        }
+        return [];
+    }
+
     async unlinkFilter(fieldName, recordId) {
         const info = this.meta.filtersInfo[fieldName];
+        const section = this.data.filterSections[fieldName];
+        if (section) {
+            // remove the filter directly, to provide a direct feedback to the user
+            this.keepLast.add(Promise.resolve());
+            section.filters = section.filters.filter((f) => f.recordId !== recordId);
+        }
         if (info && info.writeResModel) {
             await this.orm.unlink(info.writeResModel, [recordId]);
-            await this.load();
+            await this.debouncedLoad();
         }
     }
     async unlinkRecord(recordId) {
         await this.orm.unlink(this.meta.resModel, [recordId]);
         await this.load();
     }
-    async updateFilters(fieldName, filters) {
-        const section = this.data.filterSections[fieldName];
-        if (section) {
-            for (const value in filters) {
-                const active = filters[value];
-                const filter = section.filters.find((filter) => `${filter.value}` === value);
-                if (filter) {
-                    filter.active = active;
-                    const info = this.meta.filtersInfo[fieldName];
-                    if (
-                        filter.recordId &&
-                        info &&
-                        info.writeFieldName &&
-                        info.writeResModel &&
-                        info.filterFieldName
-                    ) {
-                        const data = {
-                            [info.filterFieldName]: active,
-                        };
-                        await this.orm.write(info.writeResModel, [filter.recordId], data);
-                    }
-                }
+
+    async unlinkRecords(recordsId) {
+        if (recordsId.length) {
+            await this.orm.unlink(this.meta.resModel, recordsId);
+            await this.load();
+        }
+    }
+
+    async updateFilters(fieldName, filters, active) {
+        // update filters directly, to provide a direct feedback to the user
+        this.keepLast.add(Promise.resolve());
+        for (const filter of filters) {
+            filter.active = active;
+        }
+        const info = this.meta.filtersInfo[fieldName];
+        if (info && info.writeFieldName && info.writeResModel && info.filterFieldName) {
+            const userFilter = filters.find((f) => f.type === "user");
+            if (userFilter) {
+                userFilter.active = active;
+            }
+            const filterIds = filters.filter((f) => f.type === "record").map((f) => f.recordId);
+            if (filterIds) {
+                const data = {
+                    [info.filterFieldName]: active,
+                };
+                const context = this.meta.context;
+                await this.orm.write(info.writeResModel, filterIds, data, { context });
             }
         }
-        await this.load();
+        await this.debouncedLoad();
     }
     async updateRecord(record, options = {}) {
         const rawRecord = this.buildRawRecord(record, options);
         delete rawRecord.name; // name is immutable.
         await this.orm.write(this.meta.resModel, [record.id], rawRecord, {
-            context: { from_ui: true },
+            context: this.meta.context,
         });
         await this.load();
     }
 
     //--------------------------------------------------------------------------
+    getAllDayDates(start, end) {
+        return [start.set({ hours: 7 }), end.set({ hours: 19 })];
+    }
 
     buildRawRecord(partialRecord, options = {}) {
         const data = {};
@@ -218,18 +353,17 @@ export class CalendarModel extends Model {
                 end = start;
             } else {
                 // in week mode or day mode, convert allday event to event
-                end = start.plus({ hours: 2 });
+                end = start.plus({ hours: options.duration_hour || 1 });
             }
         }
 
-        const isDateEvent = this.fields[this.meta.fieldMapping.date_start].type === "date";
+        const isDateEvent = this.dateStartType === "date";
         // An "all day" event without the "all_day" option is not considered
         // as a 24h day. It's just a part of the day (by default: 7h-19h).
         if (partialRecord.isAllDay) {
             if (!this.hasAllDaySlot && !isDateEvent && !partialRecord.id) {
                 // default hours in the user's timezone
-                start = start.set({ hours: 7 });
-                end = end.set({ hours: 19 });
+                [start, end] = this.getAllDayDates(start, end);
             }
         }
 
@@ -238,17 +372,14 @@ export class CalendarModel extends Model {
         }
 
         data[this.meta.fieldMapping.date_start] =
-            (partialRecord.isAllDay && this.hasAllDaySlot
-                ? "date"
-                : this.fields[this.meta.fieldMapping.date_start].type) === "date"
+            (partialRecord.isAllDay && this.hasAllDaySlot ? "date" : this.dateStartType) === "date"
                 ? serializeDate(start)
                 : serializeDateTime(start);
 
         if (this.meta.fieldMapping.date_stop) {
             data[this.meta.fieldMapping.date_stop] =
-                (partialRecord.isAllDay && this.hasAllDaySlot
-                    ? "date"
-                    : this.fields[this.meta.fieldMapping.date_start].type) === "date"
+                (partialRecord.isAllDay && this.hasAllDaySlot ? "date" : this.dateStartType) ===
+                "date"
                     ? serializeDate(end)
                     : serializeDateTime(end);
         }
@@ -292,15 +423,12 @@ export class CalendarModel extends Model {
      * @protected
      */
     async updateData(data) {
-        if (data.hasCreateRight === null) {
-            data.hasCreateRight = await this.orm.call(this.meta.resModel, "check_access_rights", [
-                "create",
-                false,
-            ]);
-        }
         data.range = this.computeRange();
+        let unusualDaysProm;
         if (this.meta.showUnusualDays) {
-            data.unusualDays = await this.loadUnusualDays(data);
+            unusualDaysProm = this.loadUnusualDays(data).then((unusualDays) => {
+                data.unusualDays = unusualDays;
+            });
         }
 
         const { sections, dynamicFiltersInfo } = await this.loadFilters(data);
@@ -314,14 +442,39 @@ export class CalendarModel extends Model {
         Object.assign(data.filterSections, dynamicSections);
 
         // Remove records that don't match dynamic filters
-        for (const [recordId, record] of Object.entries(data.records)) {
-            for (const [fieldName, filterInfo] of Object.entries(dynamicSections)) {
-                for (const filter of filterInfo.filters) {
-                    const rawValue = record.rawRecord[fieldName];
-                    const value = Array.isArray(rawValue) ? rawValue[0] : rawValue;
-                    if (filter.value === value && !filter.active) {
-                        delete data.records[recordId];
-                    }
+        for (const [fieldName, filterInfo] of Object.entries(dynamicSections)) {
+            const field = this.meta.fields[fieldName];
+            if (!field) {
+                continue;
+            }
+            const inactiveFilters = filterInfo.filters.filter((f) => !f.active);
+            if (!inactiveFilters.length) {
+                continue;
+            }
+            for (const [recordId, record] of Object.entries(data.records)) {
+                const rawValue = record.rawRecord[fieldName];
+                let remove = false;
+                if (["many2many", "one2many"].includes(field.type)) {
+                    const inactiveFilterVals = inactiveFilters.map((filter) => filter.value);
+                    remove = intersection(rawValue, inactiveFilterVals).length === rawValue.length;
+                } else {
+                    const recordValue = Array.isArray(rawValue) ? rawValue[0] : rawValue;
+                    remove = inactiveFilters.some((filter) => filter.value === recordValue);
+                }
+                if (remove) {
+                    delete data.records[recordId];
+                }
+            }
+        }
+
+        await unusualDaysProm;
+
+        // Compute aggregate values
+        if (this.aggregate) {
+            for (const [fieldName, { filters }] of Object.entries(data.filterSections)) {
+                const aggregates = this.computeAggregatedValues(fieldName, data);
+                for (const filter of filters) {
+                    filter.aggregatedValue = aggregates[filter.value] || 0;
                 }
             }
         }
@@ -344,9 +497,9 @@ export class CalendarModel extends Model {
             end = end.endOf(scale);
         }
 
-        if (["week", "month"].includes(scale)) {
-            const weekday = start.weekday < firstDayOfWeek ? firstDayOfWeek - 7 : firstDayOfWeek;
-            start = start.set({ weekday });
+        if (scale === "week" || (scale === "month" && this.monthOverflow)) {
+            const currentWeekOffset = (start.weekday - firstDayOfWeek + 7) % 7;
+            start = start.minus({ days: currentWeekOffset });
             end = start.plus({ weeks: scale === "week" ? 1 : 6, days: -1 });
         }
 
@@ -358,6 +511,29 @@ export class CalendarModel extends Model {
 
     //--------------------------------------------------------------------------
 
+    /**
+     * @param {string} fieldName
+     * @param {Object} [data=this.data]
+     * @returns Object
+     */
+    computeAggregatedValues(fieldName, data = this.data) {
+        const records = Object.values(data.records);
+        const fieldType = this.meta.fields[fieldName].type;
+        const groups = groupBy(records, ({ rawRecord }) => {
+            const rawValue = rawRecord[fieldName];
+            // FIXME: many2many not supported, but not supported for filters either
+            return fieldType === "many2one" ? rawValue?.[0] || false : rawValue;
+        });
+        const aggregates = {};
+        const [aggregateField, aggregator] = this.aggregate.split(":");
+        for (const group in groups) {
+            const values = groups[group].map(({ rawRecord }) => rawRecord[aggregateField]);
+            aggregates[group] = formatFloat(computeAggregatedValue(values, aggregator), {
+                trailingZeros: false,
+            });
+        }
+        return aggregates;
+    }
     /**
      * @protected
      */
@@ -378,27 +554,22 @@ export class CalendarModel extends Model {
         const avoidValues = {};
 
         for (const [fieldName, filterSection] of Object.entries(data.filterSections)) {
-            // Skip "all" filters because they do not affect the domain
-            const filterAll = filterSection.filters.find((f) => f.type === "all");
-            if (!(filterAll && filterAll.active)) {
-                const filterSectionInfo = this.meta.filtersInfo[fieldName];
-
-                // Loop over subfilters to complete authorizedValues
-                for (const filter of filterSection.filters) {
-                    if (filterSectionInfo.writeResModel) {
-                        if (!authorizedValues[fieldName]) {
-                            authorizedValues[fieldName] = [];
-                        }
-                        if (filter.active) {
-                            authorizedValues[fieldName].push(filter.value);
-                        }
-                    } else {
-                        if (!filter.active) {
-                            if (!avoidValues[fieldName]) {
-                                avoidValues[fieldName] = [];
-                            }
-                            avoidValues[fieldName].push(filter.value);
+            const filterSectionInfo = this.meta.filtersInfo[fieldName];
+            // Loop over subfilters to complete authorizedValues
+            for (const filter of filterSection.filters) {
+                if (filterSectionInfo.writeResModel) {
+                    if (!authorizedValues[fieldName]) {
+                        authorizedValues[fieldName] = [];
+                    }
+                    if (filter.active) {
+                        authorizedValues[fieldName].push(filter.value);
+                    }
+                } else {
+                    if (!filter.active) {
+                        if (!avoidValues[fieldName]) {
+                            avoidValues[fieldName] = [];
                         }
+                        avoidValues[fieldName].push(filter.value);
                     }
                 }
             }
@@ -421,8 +592,9 @@ export class CalendarModel extends Model {
      */
     computeRangeDomain(data) {
         const { fieldMapping } = this.meta;
-        const formattedEnd = serializeDateTime(data.range.end);
-        const formattedStart = serializeDateTime(data.range.start);
+        const serializeFn = this.dateStartType === "date" ? serializeDate : serializeDateTime;
+        const formattedEnd = serializeFn(data.range.end);
+        const formattedStart = serializeFn(data.range.start);
 
         const domain = [[fieldMapping.date_start, "<=", formattedEnd]];
         if (fieldMapping.date_stop) {
@@ -448,7 +620,7 @@ export class CalendarModel extends Model {
      * @protected
      */
     async loadUnusualDays(data) {
-        const unusualDays = await this.fetchUnusualDays(data);
+        const unusualDays = await this._unusualDaysCache.read(data);
         return Object.entries(unusualDays)
             .filter((entry) => entry[1])
             .map((entry) => entry[0]);
@@ -460,8 +632,13 @@ export class CalendarModel extends Model {
      * @protected
      */
     fetchRecords(data) {
-        const { fieldNames, resModel } = this.meta;
-        return this.orm.searchRead(resModel, this.computeDomain(data), fieldNames);
+        const { context, fieldNames, resModel } = this.meta;
+        return this.orm.searchRead(
+            resModel,
+            this.computeDomain(data),
+            [...new Set([...fieldNames, ...Object.keys(this.meta.activeFields)])],
+            { context }
+        );
     }
     /**
      * @protected
@@ -479,7 +656,7 @@ export class CalendarModel extends Model {
      * @param {Record<string, any>} rawRecord
      */
     normalizeRecord(rawRecord) {
-        const { fields, fieldMapping, isTimeHidden, scale } = this.meta;
+        const { fields, fieldMapping, isTimeHidden } = this.meta;
 
         const startType = fields[fieldMapping.date_start].type;
         const isAllDay =
@@ -511,7 +688,6 @@ export class CalendarModel extends Model {
 
         const showTime =
             !(fieldMapping.all_day && rawRecord[fieldMapping.all_day]) &&
-            scale !== "year" &&
             startType !== "date" &&
             start.day === end.day;
 
@@ -533,18 +709,34 @@ export class CalendarModel extends Model {
             isHatched: rawRecord["is_hatched"] || false,
             isStriked: rawRecord["is_striked"] || false,
             isTimeHidden: isTimeHidden || !showTime,
+            isMonth: this.meta.scale === "month",
+            isSmall: this.env.isSmall,
             rawRecord,
         };
     }
 
+    /**
+     * @protected
+     */
+    addFilterFields(record, filterInfo) {
+        return {
+            colorIndex: record.colorIndex,
+        };
+    }
     //--------------------------------------------------------------------------
 
     /**
      * @protected
      */
     fetchFilters(resModel, fieldNames) {
-        return this.orm.searchRead(resModel, [["user_id", "=", this.user.userId]], fieldNames);
+        return this.orm.searchRead(resModel, [["user_id", "=", user.userId]], fieldNames);
     }
+
+    getLocalStorageScale() {
+        const localScaleId = browser.localStorage.getItem(this.storageKey);
+        return this.meta.scales.includes(localScaleId) ? localScaleId : this.meta.scale;
+    }
+
     /**
      * @protected
      */
@@ -552,18 +744,21 @@ export class CalendarModel extends Model {
         const previousSections = data.filterSections;
         const sections = {};
         const dynamicFiltersInfo = {};
+        const proms = [];
         for (const [fieldName, filterInfo] of Object.entries(this.meta.filtersInfo)) {
             const previousSection = previousSections[fieldName];
             if (filterInfo.writeResModel) {
-                sections[fieldName] = await this.loadFilterSection(
-                    fieldName,
-                    filterInfo,
-                    previousSection
+                const prom = this.loadFilterSection(fieldName, filterInfo, previousSection).then(
+                    (result) => {
+                        sections[fieldName] = result;
+                    }
                 );
+                proms.push(prom);
             } else {
                 dynamicFiltersInfo[fieldName] = { filterInfo, previousSection };
             }
         }
+        await Promise.all(proms);
         return { sections, dynamicFiltersInfo };
     }
     /**
@@ -591,9 +786,6 @@ export class CalendarModel extends Model {
             );
         }
 
-        const previousAllFilter = previousFilters.find((f) => f.type === "all");
-        filters.push(this.makeFilterAll(previousAllFilter, isUserOrPartner));
-
         return {
             label: filterInfo.label,
             fieldName,
@@ -607,8 +799,8 @@ export class CalendarModel extends Model {
                 field: writeFieldName,
                 model: writeResModel,
             },
-            canCollapse: filters.length > 2,
             canAddFilter: !!filterInfo.writeResModel,
+            context: makeContext([filterInfo.context, this.meta.context]),
         };
     }
     /**
@@ -616,14 +808,19 @@ export class CalendarModel extends Model {
      */
     async loadDynamicFilters(data, filtersInfo) {
         const sections = {};
+        const proms = [];
         for (const [fieldName, { filterInfo, previousSection }] of Object.entries(filtersInfo)) {
-            sections[fieldName] = await this.loadDynamicFilterSection(
+            const prom = this.loadDynamicFilterSection(
                 data,
                 fieldName,
                 filterInfo,
                 previousSection
-            );
+            ).then((result) => {
+                sections[fieldName] = result;
+            });
+            proms.push(prom);
         }
+        await Promise.all(proms);
         return sections;
     }
     /**
@@ -645,26 +842,49 @@ export class CalendarModel extends Model {
                     filters.push({
                         id: value,
                         [fieldName]: rawValue,
-                        colorIndex: record.colorIndex,
+                        ...this.addFilterFields(record, filterInfo),
                     });
                 }
             }
             return filters;
         }, []);
 
-        const { colorFieldName } = filterInfo;
-        const shouldFetchColor =
-            colorFieldName &&
-            `${fieldName}.${colorFieldName}` !== fields[fieldMapping.color].related;
+        const isX2Many = ["many2many", "one2many"].includes(field.type);
+
+        const relatedIds = rawFilters.map((f) => f.id).filter((id) => id);
         let rawColors = [];
-        if (shouldFetchColor) {
-            const relatedIds = rawFilters.map(({ id }) => id);
-            if (relatedIds.length) {
-                rawColors = await this.orm.searchRead(
+        if (relatedIds.length > 0 && field.relation) {
+            const fieldsToFetch = [];
+            const { colorFieldName } = filterInfo;
+            const shouldFetchColor =
+                colorFieldName &&
+                (!fieldMapping.color ||
+                    `${fieldName}.${colorFieldName}` !== fields[fieldMapping.color].related);
+            if (shouldFetchColor) {
+                fieldsToFetch.push(colorFieldName);
+            }
+            if (isX2Many) {
+                fieldsToFetch.push("display_name");
+            }
+            if (fieldsToFetch.length > 0) {
+                const records = await this.orm.searchRead(
                     field.relation,
                     [["id", "in", relatedIds]],
-                    [colorFieldName]
+                    fieldsToFetch
                 );
+                if (isX2Many) {
+                    const nameById = Object.fromEntries(records.map((r) => [r.id, r.display_name]));
+                    for (const rawFilter of rawFilters) {
+                        const id = rawFilter.id;
+                        if (!id || !nameById[id]) {
+                            continue;
+                        }
+                        rawFilter[fieldName] = [id, nameById[id]];
+                    }
+                }
+                if (shouldFetchColor) {
+                    rawColors = records;
+                }
             }
         }
 
@@ -694,7 +914,6 @@ export class CalendarModel extends Model {
                 field: filterInfo.writeFieldName,
                 model: filterInfo.writeResModel,
             },
-            canCollapse: filters.length > 2,
             canAddFilter: !!filterInfo.writeResModel,
         };
     }
@@ -706,7 +925,8 @@ export class CalendarModel extends Model {
         const rawValue = rawFilter[fieldName];
         const value = Array.isArray(rawValue) ? rawValue[0] : rawValue;
         const field = fields[fieldName];
-        const formatter = registry.category("formatters").get(field.type);
+        const isX2Many = ["many2many", "one2many"].includes(field.type);
+        const formatter = registry.category("formatters").get(isX2Many ? "many2one" : field.type);
 
         const { colorFieldName } = filterInfo;
         const colorField = fields[fieldMapping.color];
@@ -728,7 +948,7 @@ export class CalendarModel extends Model {
             type: "dynamic",
             recordId: null,
             value,
-            label: formatter(rawValue, { field }) || _t("Undefined"),
+            label: formatter(rawValue, { field }) || this.defaultFilterLabel,
             active: previousFilter ? previousFilter.active : true,
             canRemove: false,
             colorIndex,
@@ -761,10 +981,10 @@ export class CalendarModel extends Model {
         const colorIndex = Array.isArray(colorValue) ? colorValue[0] : colorValue;
 
         let active = false;
-        if (previousFilter) {
-            active = previousFilter.active;
-        } else if (filterFieldName) {
+        if (filterFieldName) {
             active = rawRecord[filterFieldName];
+        } else if (previousFilter) {
+            active = previousFilter.active;
         }
         return {
             type: "record",
@@ -783,7 +1003,7 @@ export class CalendarModel extends Model {
     makeFilterUser(filterInfo, previousFilter, fieldName, rawRecords) {
         const field = this.meta.fields[fieldName];
         const userFieldName = field.relation === "res.partner" ? "partnerId" : "userId";
-        const value = this.user[userFieldName];
+        const value = user[userFieldName];
 
         let colorIndex = value;
         const rawRecord = rawRecords.find((r) => r[filterInfo.writeFieldName][0] === value);
@@ -796,29 +1016,11 @@ export class CalendarModel extends Model {
             type: "user",
             recordId: null,
             value,
-            label: this.user.name,
+            label: user.name,
             active: previousFilter ? previousFilter.active : true,
             canRemove: false,
             colorIndex,
             hasAvatar: !!value,
         };
     }
-    /**
-     * @protected
-     */
-    makeFilterAll(previousAllFilter, isUserOrPartner) {
-        return {
-            type: "all",
-            recordId: null,
-            value: "all",
-            label: isUserOrPartner
-                ? this.env._t("Everybody's calendars")
-                : this.env._t("Everything"),
-            active: previousAllFilter ? previousAllFilter.active : false,
-            canRemove: false,
-            colorIndex: null,
-            hasAvatar: false,
-        };
-    }
 }
-CalendarModel.services = ["user"];
