@@ -1,28 +1,43 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
+from . import common
+from . import test_assets
 from . import test_attachment
 from . import test_auth_signup_uninvited
-from . import test_automatic_editor
 from . import test_base_url
+from . import test_client_action
 from . import test_configurator
 from . import test_controllers
 from . import test_converter
 from . import test_crawl
+from . import test_custom_snippets
 from . import test_disable_unused_snippets_assets
 from . import test_fuzzy
 from . import test_get_current_website
+from . import test_grid_layout
+from . import test_http_endpoint
+from . import test_iap
+from . import test_import_files
+from . import test_ir_asset
 from . import test_lang_url
 from . import test_menu
+from . import test_multi_website
 from . import test_page
+from . import test_page_manager
 from . import test_performance
 from . import test_qweb
+from . import test_redirect
 from . import test_res_users
+from . import test_sitemap
+from . import test_skip_website_configurator
 from . import test_snippets
 from . import test_theme
 from . import test_ui
+from . import test_unsplash_beacon
 from . import test_views
 from . import test_views_inherit_module_update
 from . import test_website_favicon
 from . import test_website_form_editor
 from . import test_website_reset_password
 from . import test_website_visitor
+from . import test_website_technical_page
+from . import test_website_website_builder_assets_bundle
