@@ -1,9 +1,9 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 from lxml.html import document_fromstring
 
 import odoo.tests
+from odoo.exceptions import ValidationError
 
 
 class TestUrlCommon(odoo.tests.HttpCase):
@@ -85,7 +85,7 @@ class TestBaseUrl(TestUrlCommon):
         self._assertCanonical('/?debug=1', self.website.get_base_url() + '/')
         self._assertCanonical('/a-page', self.website.get_base_url() + '/a-page')
         self._assertCanonical('/en_US', self.website.get_base_url() + '/')
-        self._assertCanonical('/fr_FR', self.website.get_base_url() + '/fr/')
+        self._assertCanonical('/fr_FR', self.website.get_base_url() + '/fr')
 
 
 @odoo.tests.tagged('-at_install', 'post_install')
@@ -149,3 +149,11 @@ class TestGetBaseUrl(odoo.tests.TransactionCase):
         with self.assertRaises(ValueError):
             # if more than one record, an error we should be raised
             Attachment.search([], limit=2).get_base_url()
+
+    def test_03_invalid_website_domain(self):
+        website = self.env['website'].create({
+            'name': 'Website Test 2',
+        })
+
+        with self.assertRaises(ValidationError):
+            website.write({'domain': 'https://my-website.net['})
