@@ -1,26 +1,27 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
-from odoo import api, SUPERUSER_ID, _
+
 from . import controllers
 from . import models
-from . import wizard
 from . import report
 
-def _post_init_hook(cr, registry):
-    env = api.Environment(cr, SUPERUSER_ID, {})
+
+def _post_init_hook(env):
     terms_conditions = env['ir.config_parameter'].get_param('account.use_invoice_terms')
     if not terms_conditions:
         env['ir.config_parameter'].set_param('account.use_invoice_terms', True)
     companies = env['res.company'].search([])
     for company in companies:
         company.terms_type = 'html'
+    env['website'].search([]).auth_signup_uninvited = 'b2c'
 
+    existing_websites = env['website'].search([])
+    for website in existing_websites:
+        website._create_checkout_steps()
 
-def uninstall_hook(cr, registry):
+def uninstall_hook(env):
     ''' Need to reenable the `product` pricelist multi-company rule that were
         disabled to be 'overridden' for multi-website purpose
     '''
-    env = api.Environment(cr, SUPERUSER_ID, {})
     pl_rule = env.ref('product.product_pricelist_comp_rule', raise_if_not_found=False)
     pl_item_rule = env.ref('product.product_pricelist_item_comp_rule', raise_if_not_found=False)
     multi_company_rules = pl_rule or env['ir.rule']
