@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 {
@@ -13,15 +12,21 @@ Allow the user to select if he wants to receive email notifications when a produ
         'website_sale_wishlist',
     ],
     'data': [
-        'views/templates.xml',
-        'data/template_email.xml',
-        'data/ir_cron_data.xml',
+        'views/website_sale_stock_wishlist_templates.xml',
     ],
     'assets': {
         'web.assets_frontend': [
-            'website_sale_stock_wishlist/static/src/**/*',
+            'website_sale_stock_wishlist/static/src/interactions/**/*',
+            'website_sale_stock_wishlist/static/src/scss/**/*',
+            'website_sale_stock_wishlist/static/src/xml/**/*',
+            (
+                'before',
+                'website_sale/static/src/interactions/website_sale.js',
+                'website_sale_stock_wishlist/static/src/js/variant_mixin.js',
+            ),
         ],
     },
     'auto_install': True,
+    'author': 'Odoo S.A.',
     'license': 'LGPL-3',
 }
