@@ -1,13 +1,10 @@
-/** @odoo-module **/
-
-import core from 'web.core';
-import publicWidget from 'web.public.widget';
-
-var _t = core._t;
+import { renderToElement } from "@web/core/utils/render";
+import publicWidget from '@web/legacy/js/public/public_widget';
+import { _t } from "@web/core/l10n/translation";
+import { rpc } from "@web/core/network/rpc";
 
 var CourseJoinWidget = publicWidget.Widget.extend({
     template: 'slide.course.join',
-    xmlDependencies: ['/website_slides/static/src/xml/slide_course_join.xml'],
     events: {
         'click .o_wslides_js_course_join_link': '_onClickJoin',
     },
@@ -19,10 +16,18 @@ var CourseJoinWidget = publicWidget.Widget.extend({
      * @param {Object} parent
      * @param {Object} options
      * @param {Object} options.channel slide.channel information
-     * @param {boolean} options.isMember whether current user is member or not
-     * @param {boolean} options.publicUser whether current user is public or not
+     * @param {boolean} options.isMember whether current user is enrolled
+     * @param {boolean} options.isMemberOrInvited whether current user is at least invited
+     * @param {string} options.inviteHash hash of the invited attendee. Needed to grant
+     *   access to a course preview / to identify.
+     * @param {integer} options.invitePartnerId id of partner of invited attendee if any.
+     *   Also needed to access course preview / to identify.
+     * @param {boolean} options.invitePreview whether the course is rendered as a preview.
+     *   This is true when an invited attendee is on the course while unlogged.
+     * @param {boolean} options.isPartnerWithoutUser whether invited partner has users. Used
+     *   to redirect properly to sign up / log in.
      * @param {string} [options.joinMessage] the message to use for the simple join case
-     *   when the course if free and the user is logged in, defaults to "Join Course".
+     *   when the course is free and the user is logged in, defaults to "Join this Course".
      * @param {Promise} [options.beforeJoin] a promise to execute before we redirect to
      *   another url within the join process (login / buy course / ...)
      * @param {function} [options.afterJoin] a callback function called after the user has
@@ -32,9 +37,14 @@ var CourseJoinWidget = publicWidget.Widget.extend({
         this._super.apply(this, arguments);
         this.channel = options.channel;
         this.isMember = options.isMember;
+        this.isMemberOrInvited = options.isMemberOrInvited;
+        this.inviteHash = options.inviteHash;
+        this.invitePartnerId = options.invitePartnerId;
+        this.invitePreview = options.invitePreview;
+        this.isPartnerWithoutUser = options.isPartnerWithoutUser;
         this.publicUser = options.publicUser;
-        this.joinMessage = options.joinMessage || _t('Join Course'),
-        this.beforeJoin = options.beforeJoin || Promise.resolve();
+        this.joinMessage = options.joinMessage || _t('Join this Course');
+        this.beforeJoin = options.beforeJoin || function () {return Promise.resolve();};
         this.afterJoin = options.afterJoin || function () {document.location.reload();};
     },
 
@@ -49,10 +59,15 @@ var CourseJoinWidget = publicWidget.Widget.extend({
     _onClickJoin: function (ev) {
         ev.preventDefault();
 
+        if (this.invitePreview || (this.channel.channelEnroll === 'invite' && this.isMemberOrInvited)) {
+            this.joinChannel(this.channel.channelId);
+            return;
+        }
+
         if (this.channel.channelEnroll !== 'invite') {
             if (this.publicUser) {
                 this.beforeJoin().then(this._redirectToLogin.bind(this));
-            } else if (!this.isMember && this.channel.channelEnroll === 'public') {
+            } else if (!this.isMember) {
                 this.joinChannel(this.channel.channelId);
             }
         }
@@ -76,9 +91,9 @@ var CourseJoinWidget = publicWidget.Widget.extend({
                 url += '?fullscreen=1';
             }
         } else {
-            url = `/slides/${this.channel.channelId}`;
+            url = `/slides/${encodeURIComponent(this.channel.channelId)}`;
         }
-        document.location = _.str.sprintf('/web/login?redirect=%s', encodeURIComponent(url));
+        document.location = `/web/login?redirect=${encodeURIComponent(url)}`;
     },
 
     /**
@@ -89,6 +104,7 @@ var CourseJoinWidget = publicWidget.Widget.extend({
     _popoverAlert: function ($el, message) {
         $el.popover({
             trigger: 'focus',
+            delay: {'hide': 300},
             placement: 'bottom',
             container: 'body',
             html: true,
@@ -107,22 +123,20 @@ var CourseJoinWidget = publicWidget.Widget.extend({
      */
     joinChannel: function (channelId) {
         var self = this;
-        this._rpc({
-            route: '/slides/channel/join',
-            params: {
-                channel_id: channelId,
-            },
+        rpc('/slides/channel/join', {
+            channel_id: channelId,
         }).then(function (data) {
             if (!data.error) {
                 self.afterJoin();
             } else {
                 if (data.error === 'public_user') {
-                    var message = _t('Please <a href="/web/login?redirect=%s">login</a> to join this course');
-                    var signupAllowed = data.error_signup_allowed || false;
-                    if (signupAllowed) {
-                        message = _t('Please <a href="/web/signup?redirect=%s">create an account</a> to join this course');
-                    }
-                    self._popoverAlert(self.$el, _.str.sprintf(message, (document.URL)));
+                    const popupContent = renderToElement('slide.course.join.popupContent', {
+                        channelId: channelId,
+                        courseUrl: encodeURIComponent(document.URL),
+                        errorSignupAllowed: data.error_signup_allowed,
+                        widget: self,
+                    });
+                    self._popoverAlert(self.$el, popupContent);
                 } else if (data.error === 'join_done') {
                     self._popoverAlert(self.$el, _t('You have already joined this channel'));
                 } else {
@@ -144,7 +158,17 @@ publicWidget.registry.websiteSlidesCourseJoin = publicWidget.Widget.extend({
         var self = this;
         var proms = [this._super.apply(this, arguments)];
         var data = self.$el.data();
-        var options = {channel: {channelEnroll: data.channelEnroll, channelId: data.channelId}};
+        var options = {
+            channel: {
+                channelEnroll: data.channelEnroll,
+                channelId: data.channelId
+            },
+            inviteHash: data.inviteHash,
+            invitePartnerId: data.invitePartnerId,
+            invitePreview: data.invitePreview,
+            isMemberOrInvited: data.isMemberOrInvited,
+            isPartnerWithoutUser: data.isPartnerWithoutUser
+        };
         $('.o_wslides_js_course_join').each(function () {
             proms.push(new CourseJoinWidget(self, options).attachTo($(this)));
         });
