@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
 RELEASE_LEVELS = [ALPHA, BETA, RELEASE_CANDIDATE, FINAL] = ['alpha', 'beta', 'candidate', 'final']
-RELEASE_LEVELS_DISPLAY = {ALPHA: ALPHA,
-                          BETA: BETA,
+RELEASE_LEVELS_DISPLAY = {ALPHA: 'a',
+                          BETA: 'b',
                           RELEASE_CANDIDATE: 'rc',
                           FINAL: ''}
 
@@ -12,9 +11,10 @@ RELEASE_LEVELS_DISPLAY = {ALPHA: ALPHA,
 # properly comparable using normal operators, for example:
 #  (6,1,0,'beta',0) < (6,1,0,'candidate',1) < (6,1,0,'candidate',2)
 #  (6,1,0,'candidate',2) < (6,1,0,'final',0) < (6,1,2,'final',0)
-version_info = (16, 0, 0, FINAL, 0, '')
-version = '.'.join(str(s) for s in version_info[:2]) + RELEASE_LEVELS_DISPLAY[version_info[3]] + str(version_info[4] or '') + version_info[5]
+# NOTE: during release, the MAJOR version can become an arbitrary string ('saas~xx')
+version_info = (19, 0, 0, FINAL, 0, '')
 series = serie = major_version = '.'.join(str(s) for s in version_info[:2])
+version = series + RELEASE_LEVELS_DISPLAY[version_info[3]] + str(version_info[4] or '') + version_info[5]
 
 product_name = 'Odoo'
 description = 'Odoo Server'
@@ -36,4 +36,10 @@ license = 'LGPL-3'
 
 nt_service_name = "odoo-server-" + series.replace('~','-')
 
-version += '-20230929'
+MIN_PY_VERSION = (3, 10)
+MAX_PY_VERSION = (3, 13)
+MIN_PG_VERSION = 13
+
+version += '-20251208'
+
+repos_heads = {'odoo': '338655e2abdcfdecaa076e6260f985ed198189c0', 'enterprise': 'f719153d6fa611e294848108afc72c7ee39f7e35', 'design-themes': '209c22a3d6103e6be90919b2414875e0a0861e16'}
