@@ -1,8 +1,8 @@
 """
 This code is what let us use ES6-style modules in odoo.
-Classic Odoo modules are composed of a top-level :samp:`odoo.define({name},{body_function})` call.
+Classic Odoo modules are composed of a top-level :samp:`odoo.define({name},{dependencies},{body_function})` call.
 This processor will take files starting with an `@odoo-module` annotation (in a comment) and convert them to classic modules.
-If any file has the /** odoo-module */ on top of it, it will get processed by this class.
+If any file has the ``/** odoo-module */`` on top of it, it will get processed by this class.
 It performs several operations to get from ES6 syntax to the usual odoo one with minimal changes.
 This is done on the fly, this not a pre-processing tool.
 
@@ -14,6 +14,8 @@ the original source need to be supported by the browsers.
 import re
 from functools import partial
 
+from odoo.tools.misc import OrderedSet
+
 
 def transpile_javascript(url, content):
     """
@@ -25,24 +27,28 @@ def transpile_javascript(url, content):
     """
     module_path = url_to_module_path(url)
     legacy_odoo_define = get_aliased_odoo_define_content(module_path, content)
-
+    dependencies = OrderedSet()
     # The order of the operations does sometimes matter.
     steps = [
         convert_legacy_default_import,
         convert_basic_import,
+        convert_default_and_named_import,
+        convert_default_and_star_import,
         convert_default_import,
         convert_star_import,
         convert_unnamed_relative_import,
         convert_from_export,
         convert_star_from_export,
-        partial(convert_relative_require, url),
         remove_index,
+        partial(convert_relative_require, url, dependencies),
         convert_export_function,
         convert_export_class,
         convert_variable_export,
         convert_object_export,
         convert_default_export,
-        partial(wrap_with_odoo_define, module_path),
+        partial(wrap_with_qunit_module, url),
+        partial(wrap_with_odoo_define, module_path, dependencies),
+        partial(convert_t, url)
     ]
     for s in steps:
         content = s(content)
@@ -61,7 +67,7 @@ URL_RE = re.compile(r"""
 
 def url_to_module_path(url):
     """
-    Odoo modules each have a name. (odoo.define("<the name>", async function (require) {...});
+    Odoo modules each have a name. (odoo.define("<the name>", [<dependencies>], function (require) {...});
     It is used in to be required later. (const { something } = require("<the name>").
     The transpiler transforms the url of the file in the project to this name.
     It takes the module name and add a @ on the start of it, and map it to be the source of the static/src (or
@@ -90,13 +96,23 @@ def url_to_module_path(url):
     else:
         raise ValueError("The js file %r must be in the folder '/static/src' or '/static/lib' or '/static/test'" % url)
 
+def wrap_with_qunit_module(url, content):
+    """
+    Wraps the test file content (source code) with the QUnit.module('module_name', function() {...}).
+    """
+    if "tests" in url and re.search(r'QUnit\.(test|debug|only)\(', content):
+        match = URL_RE.match(url)
+        return f"""QUnit.module("{match["module"]}", function() {{{content}}});"""
+    else:
+        return content
 
-def wrap_with_odoo_define(module_path, content):
+def wrap_with_odoo_define(module_path, dependencies, content):
     """
     Wraps the current content (source code) with the odoo.define call.
+    It adds as a second argument the list of dependencies.
     Should logically be called once all other operations have been performed.
     """
-    return f"""odoo.define({module_path!r}, async function (require) {{
+    return f"""odoo.define({module_path!r}, {list(dependencies)}, function (require) {{
 'use strict';
 let __exports = {{}};
 {content}
@@ -110,7 +126,7 @@ EXPORT_FCT_RE = re.compile(r"""
     (?P<space>\s*)                          # space and empty line
     export\s+                               # export
     (?P<type>(async\s+)?function)\s+        # async function or function
-    (?P<identifier>\w+)                     # name the function
+    (?P<identifier>[\w$]+)                  # name of the function
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -139,7 +155,7 @@ EXPORT_CLASS_RE = re.compile(r"""
     (?P<space>\s*)                          # space and empty line
     export\s+                               # export
     (?P<type>class)\s+                      # class
-    (?P<identifier>\w+)                     # name of the class
+    (?P<identifier>[\w$]+)                  # name of the class
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -164,7 +180,7 @@ EXPORT_FCT_DEFAULT_RE = re.compile(r"""
     (?P<space>\s*)                          # space and empty line
     export\s+default\s+                     # export default
     (?P<type>(async\s+)?function)\s+        # async function or function
-    (?P<identifier>\w+)                     # name of the function
+    (?P<identifier>[\w$]+)                  # name of the function
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -193,7 +209,7 @@ EXPORT_CLASS_DEFAULT_RE = re.compile(r"""
     (?P<space>\s*)                          # space and empty line
     export\s+default\s+                     # export default
     (?P<type>class)\s+                      # class
-    (?P<identifier>\w+)                     # name of the class or the function
+    (?P<identifier>[\w$]+)                  # name of the class or the function
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -212,12 +228,51 @@ def convert_export_class_default(content):
     repl = r"""\g<space>const \g<identifier> = __exports[Symbol.for("default")] = \g<type> \g<identifier>"""
     return EXPORT_CLASS_DEFAULT_RE.sub(repl, content)
 
+
+GETTEXT_RE = re.compile(r"""
+    ^
+    \s*const\s*{
+    (?:\s*\w*\s*,)*
+    \s*(_t)\s*
+    (?:,\s*\w*\s*)*,?\s*
+    }\s*=\s*require\("@web/core/l10n/translation"\);$
+""", re.MULTILINE | re.VERBOSE)
+
+
+T_FN_RE = re.compile(r"""
+    ^
+    \s*const\s*{
+    (?:\s*\w*\s*,)*
+    \s*(appTranslateFn)\s*
+    (?:,\s*\w*\s*)*,?\s*
+    }\s*=\s*require\("@web/core/l10n/translation"\);$
+""", re.MULTILINE | re.VERBOSE)
+
+
+def convert_t(url, content):
+    if url.endswith(".test.js"):
+        return content
+
+    module_name = URL_RE.match(url)["module"]
+    has_import_of_appTranslateFn = bool(T_FN_RE.search(content))
+
+    def rename_gettext(match_):
+        if has_import_of_appTranslateFn:
+            renamed_import = match_.group(0).replace("_t", "__not_defined__")
+        else:
+            renamed_import = match_.group(0).replace("_t", "appTranslateFn")
+        renamed_import += f"""const _t = (str, ...args) => appTranslateFn(str, "{module_name}", ...args);"""
+        return renamed_import
+
+    return GETTEXT_RE.sub(rename_gettext, content)
+
+
 EXPORT_VAR_RE = re.compile(r"""
     ^
     (?P<space>\s*)              # space and empty line
     export\s+                   # export
     (?P<type>let|const|var)\s+  # let or cont or var
-    (?P<identifier>\w+)         # variable name
+    (?P<identifier>[\w$]+)      # variable name
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -243,7 +298,7 @@ EXPORT_DEFAULT_VAR_RE = re.compile(r"""
     (?P<space>\s*)              # space and empty line
     export\s+default\s+         # export default
     (?P<type>let|const|var)\s+  # let or const or var
-    (?P<identifier>\w+)\s*      # variable name
+    (?P<identifier>[\w$]+)\s*   # variable name
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -267,7 +322,7 @@ EXPORT_OBJECT_RE = re.compile(r"""
     ^
     (?P<space>\s*)                      # space and empty line
     export\s*                           # export
-    (?P<object>{[\w\s,]+})              # { a, b, c as x, ... }
+    (?P<object>{[\w$\s,]+})             # { a, b, c as x, ... }
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -293,7 +348,7 @@ EXPORT_FROM_RE = re.compile(r"""
     ^
     (?P<space>\s*)                      # space and empty line
     export\s*                           # export
-    (?P<object>{[\w\s,]+})\s*           # { a, b, c as x, ... }
+    (?P<object>{[\w$\s,]+})\s*          # { a, b, c as x, ... }
     from\s*                             # from
     (?P<path>(?P<quote>["'`])([^"'`]+)(?P=quote))   # "file path" ("some/path.js")
     """, re.MULTILINE | re.VERBOSE)
@@ -349,7 +404,7 @@ EXPORT_DEFAULT_RE = re.compile(r"""
     ^
     (?P<space>\s*)      # space and empty line
     export\s+default    # export default
-    (\s+\w+\s*=)?       # something (optional)
+    (\s+[\w$]+\s*=)?    # something (optional)
     """, re.MULTILINE | re.VERBOSE)
 
 
@@ -385,7 +440,7 @@ IMPORT_BASIC_RE = re.compile(r"""
     ^
     (?P<space>\s*)                      # space and empty line
     import\s+                           # import
-    (?P<object>{(\s*\w+\s*,?\s*)+})\s*  # { a, b, c as x, ... }
+    (?P<object>{[\s\w$,]+})\s*          # { a, b, c as x, ... }
     from\s*                             # from
     (?P<path>(?P<quote>["'`])([^"'`]+)(?P=quote))   # "file path" ("some/path")
     """, re.MULTILINE | re.VERBOSE)
@@ -412,7 +467,7 @@ IMPORT_LEGACY_DEFAULT_RE = re.compile(r"""
     ^
     (?P<space>\s*)                                      # space and empty line
     import\s+                                           # import
-    (?P<identifier>\w+)\s*                              # default variable name
+    (?P<identifier>[\w$]+)\s*                           # default variable name
     from\s*                                             # from
     (?P<path>(?P<quote>["'`])([^@\."'`][^"'`]*)(?P=quote))  # legacy alias file ("addon_name.module_name" or "some/path")
     """, re.MULTILINE | re.VERBOSE)
@@ -439,7 +494,7 @@ IMPORT_DEFAULT = re.compile(r"""
     ^
     (?P<space>\s*)                      # space and empty line
     import\s+                           # import
-    (?P<identifier>\w+)\s*              # default variable name
+    (?P<identifier>[\w$]+)\s*           # default variable name
     from\s*                             # from
     (?P<path>(?P<quote>["'`])([^"'`]+)(?P=quote))   # "file path" ("some/path")
     """, re.MULTILINE | re.VERBOSE)
@@ -460,15 +515,54 @@ def convert_default_import(content):
     return IMPORT_DEFAULT.sub(repl, content)
 
 
+IS_PATH_LEGACY_RE = re.compile(r"""(?P<quote>["'`])([^@\."'`][^"'`]*)(?P=quote)""")
+
+IMPORT_DEFAULT_AND_NAMED_RE = re.compile(r"""
+    ^
+    (?P<space>\s*)                                  # space and empty line
+    import\s+                                       # import
+    (?P<default_export>[\w$]+)\s*,\s*               # default variable name,
+    (?P<named_exports>{[\s\w$,]+})\s*                # { a, b, c as x, ... }
+    from\s*                                         # from
+    (?P<path>(?P<quote>["'`])([^"'`]+)(?P=quote))   # "file path" ("some/path")
+    """, re.MULTILINE | re.VERBOSE)
+
+
+def convert_default_and_named_import(content):
+    """
+    Transpile default and named import on one line.
+
+    .. code-block:: javascript
+
+        // before
+        import something, { a } from "some/path";
+        import somethingElse, { b } from "legacy.module";
+        // after
+        const { [Symbol.for("default")]: something, a } = require("some/path");
+        const somethingElse = require("legacy.module");
+        const { b } = somethingElse;
+    """
+    def repl(matchobj):
+        is_legacy = IS_PATH_LEGACY_RE.match(matchobj['path'])
+        new_object = matchobj["named_exports"].replace(" as ", ": ")
+        if is_legacy:
+            return f"""{matchobj['space']}const {matchobj['default_export']} = require({matchobj['path']});
+{matchobj['space']}const {new_object} = {matchobj['default_export']}"""
+        new_object = f"""{{ [Symbol.for("default")]: {matchobj['default_export']},{new_object[1:]}"""
+        return f"{matchobj['space']}const {new_object} = require({matchobj['path']})"
+    return IMPORT_DEFAULT_AND_NAMED_RE.sub(repl, content)
+
+
 RELATIVE_REQUIRE_RE = re.compile(r"""
-    require\((?P<quote>["'`])([^@"'`]+)(?P=quote)\)  # require("some/path")
-    """, re.VERBOSE)
+    ^[^/*\n]*require\((?P<quote>[\"'`])([^\"'`]+)(?P=quote)\) # require("some/path")
+    """, re.MULTILINE | re.VERBOSE)
 
 
-def convert_relative_require(url, content):
+def convert_relative_require(url, dependencies, content):
     """
     Convert the relative path contained in a 'require()'
-    to the new path system (@module/path)
+    to the new path system (@module/path).
+    Adds all modules path to dependencies.
     .. code-block:: javascript
 
         // Relative path:
@@ -485,19 +579,22 @@ def convert_relative_require(url, content):
     """
     new_content = content
     for quote, path in RELATIVE_REQUIRE_RE.findall(new_content):
+        module_path = path
         if path.startswith(".") and "/" in path:
             pattern = rf"require\({quote}{path}{quote}\)"
-            repl = f'require("{relative_path_to_module_path(url, path)}")'
+            module_path = relative_path_to_module_path(url, path)
+            repl = f'require("{module_path}")'
             new_content = re.sub(pattern, repl, new_content)
+        dependencies.add(module_path)
     return new_content
 
 
 IMPORT_STAR = re.compile(r"""
-    ^(?P<space>\s*)       # indentation
-    import\s+\*\s+as\s+   # import * as
-    (?P<identifier>\w+)   # alias
-    \s*from\s*            # from
-    (?P<path>[^;\n]+)     # path
+    ^(?P<space>\s*)         # indentation
+    import\s+\*\s+as\s+     # import * as
+    (?P<identifier>[\w$]+)  # alias
+    \s*from\s*              # from
+    (?P<path>[^;\n]+)       # path
 """, re.MULTILINE | re.VERBOSE)
 
 
@@ -516,6 +613,34 @@ def convert_star_import(content):
     return IMPORT_STAR.sub(repl, content)
 
 
+IMPORT_DEFAULT_AND_STAR = re.compile(r"""
+    ^(?P<space>\s*)                    # indentation
+    import\s+                          # import
+    (?P<default_export>[\w$]+)\s*,\s*  # default export name,
+    \*\s+as\s+                         # * as
+    (?P<named_exports_alias>[\w$]+)    # alias
+    \s*from\s*                         # from
+    (?P<path>[^;\n]+)                  # path
+""", re.MULTILINE | re.VERBOSE)
+
+
+def convert_default_and_star_import(content):
+    """
+    Transpile import star.
+
+    .. code-block:: javascript
+
+        // before
+        import something, * as name from "some/path";
+        // after
+        const name = require("some/path");
+        const something = name[Symbol.for("default")];
+    """
+    repl = r"""\g<space>const \g<named_exports_alias> = require(\g<path>);
+\g<space>const \g<default_export> = \g<named_exports_alias>[Symbol.for("default")]"""
+    return IMPORT_DEFAULT_AND_STAR.sub(repl, content)
+
+
 IMPORT_UNNAMED_RELATIVE_RE = re.compile(r"""
     ^(?P<space>\s*)     # indentation
     import\s+           # import
@@ -560,9 +685,10 @@ def remove_index(content):
 
 
 def relative_path_to_module_path(url, path_rel):
-    """
-    Convert the relative path into a module path, which is more generic and fancy.
+    """Convert the relative path into a module path, which is more generic and
+    fancy.
 
+    :param str url:
     :param path_rel: a relative path to the current url.
     :return: module path (@module/...)
     """
@@ -574,23 +700,31 @@ def relative_path_to_module_path(url, path_rel):
 
 
 ODOO_MODULE_RE = re.compile(r"""
-    \s*                                       # some starting space
-    \/(\*|\/).*\s*                            # // or /*
-    @odoo-module                              # @odoo-module
-    (\s+alias=(?P<alias>[\w.]+))?             # alias=web.AbstractAction (optional)
-    (\s+default=(?P<default>False|false|0))?  # default=False or false or 0 (optional)
+    \s*                                # starting white space
+    \/(\*|\/)                          # /* or //
+    .*                                 # any comment in between (optional)
+    @odoo-module                       # '@odoo-module' statement
+    (?P<ignore>\s+ignore)?             # module in src | tests which should not be transpiled (optional)
+    (\s+alias=(?P<alias>[^\s*]+))?     # alias (e.g. alias=web.Widget, alias=@web/../tests/utils) (optional)
+    (\s+default=(?P<default>[\w$]+))?  # no implicit default export (e.g. default=false) (optional)
 """, re.VERBOSE)
 
 
-def is_odoo_module(content):
+def is_odoo_module(url, content):
     """
     Detect if the file is a native odoo module.
     We look for a comment containing @odoo-module.
 
+    :param url:
     :param content: source code
     :return: is this a odoo module that need transpilation ?
     """
     result = ODOO_MODULE_RE.match(content)
+    if result and result['ignore']:
+        return False
+    addon = url.split('/')[1]
+    if url.startswith(f'/{addon}/static/src') or url.startswith(f'/{addon}/static/tests'):
+        return True
     return bool(result)
 
 
@@ -606,7 +740,7 @@ def get_aliased_odoo_define_content(module_path, content):
     we have a problem when we will have converted to module to ES6: its new name will be more like
     "web/chrome/abstract_action". So the require would fail !
     So we add a second small modules, an alias, as such:
-    > odoo.define("web/chrome/abstract_action", async function(require) {
+    > odoo.define("web/chrome/abstract_action", ['web.AbstractAction'], function (require) {
     >  return require('web.AbstractAction')[Symbol.for("default")];
     > });
 
@@ -636,13 +770,13 @@ def get_aliased_odoo_define_content(module_path, content):
         alias = matchobj['alias']
         if alias:
             if matchobj['default']:
-                return """\nodoo.define(`%s`, async function(require) {
+                return """\nodoo.define(`%s`, ['%s'], function (require) {
                         return require('%s');
-                        });\n""" % (alias, module_path)
+                        });\n""" % (alias, module_path, module_path)
             else:
-                return """\nodoo.define(`%s`, async function(require) {
+                return """\nodoo.define(`%s`, ['%s'], function (require) {
                         return require('%s')[Symbol.for("default")];
-                        });\n""" % (alias, module_path)
+                        });\n""" % (alias, module_path, module_path)
 
 
 def convert_as(val):
