@@ -1,22 +1,30 @@
-# -*- coding: utf-8 -*-
 # Part of Odoo. See LICENSE file for full copyright and licensing details.
 
-import datetime
-import gc
+from contextlib import nullcontext, ExitStack
+from datetime import datetime
 import json
 import logging
 import sys
 import time
 import threading
 import re
-import functools
+import tracemalloc
 
-from psycopg2 import sql
+from psycopg2 import OperationalError
 
 from odoo import tools
+from odoo.tools import SQL
+
+from .gc import disabling_gc
+
 
 _logger = logging.getLogger(__name__)
 
+# ensure we have a non patched time for profiling times when using freezegun
+real_datetime_now = datetime.now
+real_time = time.time.__call__
+real_cpu_time = time.thread_time.__call__
+
 
 def _format_frame(frame):
     code = frame.f_code
@@ -43,7 +51,7 @@ def _get_stack_trace(frame, limit_frame=None):
         stack.append(_format_frame(frame))
         frame = frame.f_back
     if frame is None and limit_frame:
-        _logger.error("Limit frame was not found")
+        _logger.runbot("Limit frame was not found")
     return list(reversed(stack))
 
 
@@ -57,7 +65,7 @@ def stack_size():
 
 
 def make_session(name=''):
-    return f'{datetime.datetime.now():%Y-%m-%d %H:%M:%S} {name}'
+    return f'{real_datetime_now():%Y-%m-%d %H:%M:%S} {name}'
 
 
 def force_hook():
@@ -84,6 +92,7 @@ class Collector:
     It defines default behaviors for creating an entry in the collector.
     """
     name = None                 # symbolic name of the collector
+    _store = name
     _registry = {}              # map collector names to their class
 
     @classmethod
@@ -110,14 +119,21 @@ class Collector:
 
     def add(self, entry=None, frame=None):
         """ Add an entry (dict) to this collector. """
-        # todo add entry count limit
         self._entries.append({
             'stack': self._get_stack_trace(frame),
             'exec_context': getattr(self.profiler.init_thread, 'exec_context', ()),
-            'start': time.time(),
+            'start': real_time(),
             **(entry or {}),
         })
 
+    def progress(self, entry=None, frame=None):
+        """ Checks if the limits were met and add to the entries"""
+        if self.profiler.entry_count_limit \
+            and self.profiler.entry_count() >= self.profiler.entry_count_limit:
+            self.profiler.end()
+
+        self.add(entry=entry,frame=frame)
+
     def _get_stack_trace(self, frame=None):
         """ Return the stack trace to be included in a given entry. """
         frame = frame or get_current_frame(self.profiler.init_thread)
@@ -136,6 +152,9 @@ class Collector:
             self._processed = True
         return self._entries
 
+    def summary(self):
+        return f"{'='*10} {self.name} {'='*10} \n Entries: {len(self._entries)}"
+
 
 class SQLCollector(Collector):
     """
@@ -153,34 +172,71 @@ class SQLCollector(Collector):
         self.profiler.init_thread.query_hooks.remove(self.hook)
 
     def hook(self, cr, query, params, query_start, query_time):
-        self.add({
+        self.progress({
             'query': str(query),
             'full_query': str(cr._format(query, params)),
             'start': query_start,
             'time': query_time,
         })
 
+    def summary(self):
+        total_time = sum(entry['time'] for entry in self._entries) or 1
+        sql_entries = ''
+        for entry in self._entries:
+            sql_entries += f"\n{'-' * 100}'\n'{entry['time']}  {'*' * int(entry['time'] / total_time * 100)}'\n'{entry['full_query']}"
+        return super().summary() + sql_entries
 
-class PeriodicCollector(Collector):
+
+class _BasePeriodicCollector(Collector):
     """
     Record execution frames asynchronously at most every `interval` seconds.
 
     :param interval (float): time to wait in seconds between two samples.
     """
-    name = 'traces_async'
+    _min_interval = 0.001  # minimum interval allowed
+    _max_interval = 5    # maximum interval allowed
+    _default_interval = 0.001
 
-    def __init__(self, interval=0.01):  # check duration. dynamic?
+    def __init__(self, interval=None):  # check duration. dynamic?
         super().__init__()
         self.active = False
-        self.frame_interval = interval
-        self.thread = threading.Thread(target=self.run)
+        self.frame_interval = interval or self._default_interval
+        self.__thread = threading.Thread(target=self.run)
         self.last_frame = None
 
+    def start(self):
+        interval = self.profiler.params.get(f'{self.name}_interval')
+        if interval:
+            self.frame_interval = min(max(float(interval), self._min_interval), self._max_interval)
+        init_thread = self.profiler.init_thread
+        if not hasattr(init_thread, 'profile_hooks'):
+            init_thread.profile_hooks = []
+        init_thread.profile_hooks.append(self.progress)
+        self.__thread.start()
+
     def run(self):
         self.active = True
-        last_time = time.time()
+        self.last_time = real_time()
         while self.active:  # maybe add a check on parent_thread state?
-            duration = time.time() - last_time
+            self.progress()
+            time.sleep(self.frame_interval)
+
+        self._entries.append({'stack': [], 'start': real_time()})  # add final end frame
+
+    def stop(self):
+        self.active = False
+        self.__thread.join()
+        self.profiler.init_thread.profile_hooks.remove(self.progress)
+
+
+class PeriodicCollector(_BasePeriodicCollector):
+
+    name = 'traces_async'
+
+    def add(self, entry=None, frame=None):
+        """ Add an entry (dict) to this collector. """
+        if self.last_frame:
+            duration = real_time() - self._last_time
             if duration > self.frame_interval * 10 and self.last_frame:
                 # The profiler has unexpectedly slept for more than 10 frame intervals. This may
                 # happen when calling a C library without releasing the GIL. In that case, the
@@ -188,32 +244,9 @@ class PeriodicCollector(Collector):
                 # the call itself does not appear in any of those frames: the duration of the call
                 # is incorrectly attributed to the last frame.
                 self._entries[-1]['stack'].append(('profiling', 0, '⚠ Profiler freezed for %s s' % duration, ''))
-                self.last_frame = None  # skip duplicate detection for the next frame.
-            self.add()
-            last_time = time.time()
-            time.sleep(self.frame_interval)
+            self.last_frame = None  # skip duplicate detection for the next frame.
+        self._last_time = real_time()
 
-        self._entries.append({'stack': [], 'start': time.time()})  # add final end frame
-
-    def start(self):
-        interval = self.profiler.params.get('traces_async_interval')
-        if interval:
-            self.frame_interval = min(max(float(interval), 0.001), 1)
-
-        init_thread = self.profiler.init_thread
-        if not hasattr(init_thread, 'profile_hooks'):
-            init_thread.profile_hooks = []
-        init_thread.profile_hooks.append(self.add)
-
-        self.thread.start()
-
-    def stop(self):
-        self.active = False
-        self.thread.join()
-        self.profiler.init_thread.profile_hooks.remove(self.add)
-
-    def add(self, entry=None, frame=None):
-        """ Add an entry (dict) to this collector. """
         frame = frame or get_current_frame(self.profiler.init_thread)
         if frame == self.last_frame:
             # don't save if the frame is exactly the same as the previous one.
@@ -223,6 +256,42 @@ class PeriodicCollector(Collector):
         super().add(entry=entry, frame=frame)
 
 
+_lock = threading.Lock()
+
+
+class MemoryCollector(_BasePeriodicCollector):
+
+    name = 'memory'
+    _store = 'others'
+    _min_interval = 0.01  # minimum interval allowed
+    _default_interval = 1
+
+    def start(self):
+        _lock.acquire()
+        tracemalloc.start()
+        super().start()
+
+    def add(self, entry=None, frame=None):
+        """ Add an entry (dict) to this collector. """
+        self._entries.append({
+            'start': real_time(),
+            'memory': tracemalloc.take_snapshot(),
+        })
+
+    def stop(self):
+        _lock.release()
+        tracemalloc.stop()
+        super().stop()
+
+    def post_process(self):
+        for i, entry in enumerate(self._entries):
+            if entry.get("memory", False):
+                entry_statistics = entry["memory"].statistics('traceback')
+                modified_entry_statistics = [{'traceback': list(statistic.traceback._frames),
+                                            'size': statistic.size} for statistic in entry_statistics]
+                self._entries[i] = {"memory_tracebacks": modified_entry_statistics, "start": entry['start']}
+
+
 class SyncCollector(Collector):
     """
     Record complete execution synchronously.
@@ -231,6 +300,8 @@ class SyncCollector(Collector):
     name = 'traces_sync'
 
     def start(self):
+        if sys.gettrace() is not None:
+            _logger.error("Cannot start SyncCollector, settrace already set: %s", sys.gettrace())
         assert not self._processed, "You cannot start SyncCollector after accessing entries."
         sys.settrace(self.hook)  # todo test setprofile, but maybe not multithread safe
 
@@ -244,7 +315,7 @@ class SyncCollector(Collector):
         if event == 'call' and _frame.f_back:
             # we need the parent frame to determine the line number of the call
             entry['parent_frame'] = _format_frame(_frame.f_back)
-        self.add(entry, frame=_frame)
+        self.progress(entry, frame=_frame)
         return self.hook
 
     def _get_stack_trace(self, frame=None):
@@ -275,53 +346,6 @@ class SyncCollector(Collector):
 
 class QwebTracker():
 
-    @classmethod
-    def wrap_render(cls, method_render):
-        @functools.wraps(method_render)
-        def _tracked_method_render(self, template, values=None, **options):
-            current_thread = threading.current_thread()
-            execution_context_enabled = getattr(current_thread, 'profiler_params', {}).get('execution_context_qweb')
-            qweb_hooks = getattr(current_thread, 'qweb_hooks', ())
-            if execution_context_enabled or qweb_hooks:
-                # To have the new compilation cached because the generated code will change.
-                # Therefore 'profile' is a key to the cache.
-                options['profile'] = True
-            return method_render(self, template, values, **options)
-        return _tracked_method_render
-
-    @classmethod
-    def wrap_compile(cls, method_compile):
-        @functools.wraps(method_compile)
-        def _tracked_compile(self, template, options):
-            if not options.get('profile'):
-                return method_compile(self, template, options)
-
-            render_template = method_compile(self, template, options)
-            def profiled_method_compile(self, values):
-                ref = options.get('ref')
-                ref_xml = options.get('ref_xml')
-                qweb_tracker = QwebTracker(ref, ref_xml, self.env.cr)
-                self = self.with_context(qweb_tracker=qweb_tracker)
-                if qweb_tracker.execution_context_enabled:
-                    with ExecutionContext(template=ref):
-                        return render_template(self, values)
-                return render_template(self, values)
-            return profiled_method_compile
-        return _tracked_compile
-
-    @classmethod
-    def wrap_compile_directive(cls, method_compile_directive):
-        @functools.wraps(method_compile_directive)
-        def _tracked_compile_directive(self, el, options, directive, indent):
-            if not options.get('profile') or directive in ('content', 'tag'):
-                return method_compile_directive(self, el, options, directive, indent)
-
-            enter = self._indent(f"self.env.context['qweb_tracker'].enter_directive({directive!r}, {el.attrib!r}, {options['last_path_node']!r})", indent)
-            leave = self._indent("self.env.context['qweb_tracker'].leave_directive()", indent)
-            code_directive = method_compile_directive(self, el, options, directive, indent)
-            return [enter, *code_directive, leave] if code_directive else []
-        return _tracked_compile_directive
-
     def __init__(self, view_id, arch, cr):
         current_thread = threading.current_thread()  # don't store current_thread on self
         self.execution_context_enabled = getattr(current_thread, 'profiler_params', {}).get('execution_context_qweb')
@@ -335,19 +359,46 @@ class QwebTracker():
     def enter_directive(self, directive, attrib, xpath):
         execution_context = None
         if self.execution_context_enabled:
-            execution_context = tools.profiler.ExecutionContext(directive=directive, xpath=xpath)
+            directive_info = {}
+            if ('t-' + directive) in attrib:
+                directive_info['t-' + directive] = repr(attrib['t-' + directive])
+            if directive == 'set':
+                if 't-value' in attrib:
+                    directive_info['t-value'] = repr(attrib['t-value'])
+                if 't-valuef' in attrib:
+                    directive_info['t-valuef'] = repr(attrib['t-valuef'])
+
+                for key in attrib:
+                    if key.startswith('t-set-') or key.startswith('t-setf-'):
+                        directive_info[key] = repr(attrib[key])
+            elif directive == 'foreach':
+                directive_info['t-as'] = repr(attrib['t-as'])
+            elif directive == 'groups' and 'groups' in attrib and not directive_info.get('t-groups'):
+                directive_info['t-groups'] = repr(attrib['groups'])
+            elif directive == 'att':
+                for key in attrib:
+                    if key.startswith('t-att-') or key.startswith('t-attf-'):
+                        directive_info[key] = repr(attrib[key])
+            elif directive == 'options':
+                for key in attrib:
+                    if key.startswith('t-options-'):
+                        directive_info[key] = repr(attrib[key])
+            elif ('t-' + directive) not in attrib:
+                directive_info['t-' + directive] = None
+
+            execution_context = tools.profiler.ExecutionContext(**directive_info, xpath=xpath)
             execution_context.__enter__()
             self.context_stack.append(execution_context)
 
         for hook in self.qweb_hooks:
             hook('enter', self.cr.sql_log_count, view_id=self.view_id, xpath=xpath, directive=directive, attrib=attrib)
 
-    def leave_directive(self):
+    def leave_directive(self, directive, attrib, xpath):
         if self.execution_context_enabled:
             self.context_stack.pop().__exit__()
 
         for hook in self.qweb_hooks:
-            hook('leave', self.cr.sql_log_count)
+            hook('leave', self.cr.sql_log_count, view_id=self.view_id, xpath=xpath, directive=directive, attrib=attrib)
 
 
 class QwebCollector(Collector):
@@ -361,27 +412,42 @@ class QwebCollector(Collector):
         self.events = []
 
         def hook(event, sql_log_count, **kwargs):
-            self.events.append((event, kwargs, sql_log_count, time.time()))
+            self.events.append((event, kwargs, sql_log_count, real_time()))
         self.hook = hook
 
     def _get_directive_profiling_name(self, directive, attrib):
         expr = ''
         if directive == 'set':
-            expr = f"t-set={repr(attrib['t-set'])}"
-            if 't-value' in attrib:
-                expr = f"{expr} t-value={repr(attrib['t-value'])}"
-            if 't-valuef' in attrib:
-                expr = f"{expr} t-valuef={repr(attrib['t-valuef'])}"
+            if 't-set' in attrib:
+                expr = f"t-set={repr(attrib['t-set'])}"
+                if 't-value' in attrib:
+                    expr += f" t-value={repr(attrib['t-value'])}"
+                if 't-valuef' in attrib:
+                    expr += f" t-valuef={repr(attrib['t-valuef'])}"
+            for key in attrib:
+                if key.startswith('t-set-') or key.startswith('t-setf-'):
+                    if expr:
+                        expr += ' '
+                    expr += f"{key}={repr(attrib[key])}"
         elif directive == 'foreach':
             expr = f"t-foreach={repr(attrib['t-foreach'])} t-as={repr(attrib['t-as'])}"
         elif directive == 'options':
             if attrib.get('t-options'):
                 expr = f"t-options={repr(attrib['t-options'])}"
-            for key in list(attrib):
+            for key in attrib:
                 if key.startswith('t-options-'):
                     expr = f"{expr}  {key}={repr(attrib[key])}"
-        elif directive and ('t-' + directive) in attrib:
+        elif directive == 'att':
+            for key in attrib:
+                if key == 't-att' or key.startswith('t-att-') or key.startswith('t-attf-'):
+                    if expr:
+                        expr += ' '
+                    expr += f"{key}={repr(attrib[key])}"
+        elif ('t-' + directive) in attrib:
             expr = f"t-{directive}={repr(attrib['t-' + directive])}"
+        else:
+            expr = f"t-{directive}"
+
         return expr
 
     def start(self):
@@ -412,19 +478,21 @@ class QwebCollector(Collector):
             last_event_time = time
             last_event_query = sql_count
 
-            if event == 'enter':
-                data = {
-                    'view_id': kwargs['view_id'],
-                    'xpath': kwargs['xpath'],
-                    'directive': self._get_directive_profiling_name(kwargs['directive'], kwargs['attrib']),
-                    'delay': 0,
-                    'query': 0,
-                }
-                results.append(data)
-                stack.append(data)
-            else:
-                assert event == "leave"
-                data = stack.pop()
+            directive = self._get_directive_profiling_name(kwargs['directive'], kwargs['attrib'])
+            if directive:
+                if event == 'enter':
+                    data = {
+                        'view_id': kwargs['view_id'],
+                        'xpath': kwargs['xpath'],
+                        'directive': directive,
+                        'delay': 0,
+                        'query': 0,
+                    }
+                    results.append(data)
+                    stack.append(data)
+                else:
+                    assert event == "leave"
+                    data = stack.pop()
 
         self.add({'results': {'archs': archs, 'data': results}})
         super().post_process()
@@ -455,7 +523,7 @@ class Profiler:
     Will save sql and async stack trace by default.
     """
     def __init__(self, collectors=None, db=..., profile_session=None,
-                 description=None, disable_gc=False, params=None):
+                 description=None, disable_gc=False, params=None, log=False):
         """
         :param db: database name to use to save results.
             Will try to define database automatically by default.
@@ -468,6 +536,8 @@ class Profiler:
         """
         self.start_time = 0
         self.duration = 0
+        self.start_cpu_time = 0
+        self.cpu_duration = 0
         self.profile_session = profile_session or make_session()
         self.description = description
         self.init_frame = None
@@ -476,6 +546,12 @@ class Profiler:
         self.disable_gc = disable_gc
         self.filecache = {}
         self.params = params or {}  # custom parameters usable by collectors
+        self.profile_id = None
+        self.log = log
+        self.sub_profilers = []
+        self.entry_count_limit = int(self.params.get("entry_count_limit",0)) # the limit could be set using a smarter way
+        self.done = False
+        self.exit_stack = ExitStack()
 
         if db is ...:
             # determine database from current thread
@@ -501,26 +577,47 @@ class Profiler:
 
     def __enter__(self):
         self.init_thread = threading.current_thread()
-        self.init_frame = get_current_frame(self.init_thread)
-        self.init_stack_trace = _get_stack_trace(self.init_frame)
+        try:
+            self.init_frame = get_current_frame(self.init_thread)
+            self.init_stack_trace = _get_stack_trace(self.init_frame)
+        except KeyError:
+            # when using thread pools (gevent) the thread won't exist in the current_frames
+            # this case is managed by http.py but will still fail when adding a profiler
+            # inside a piece of code that may be called by a longpolling route.
+            # in this case, avoid crashing the caller and disable all collectors
+            self.init_frame = self.init_stack_trace = self.collectors = []
+            self.db = self.params = None
+            message = "Cannot start profiler, thread not found. Is the thread part of a thread pool?"
+            if not self.description:
+                self.description = message
+            _logger.warning(message)
+
         if self.description is None:
             frame = self.init_frame
             code = frame.f_code
             self.description = f"{frame.f_code.co_name} ({code.co_filename}:{frame.f_lineno})"
         if self.params:
             self.init_thread.profiler_params = self.params
-        if self.disable_gc and gc.isenabled():
-            gc.disable()
-        self.start_time = time.time()
+        if self.disable_gc:
+            self.exit_stack.enter_context(disabling_gc())
+        self.start_time = real_time()
+        self.start_cpu_time = real_cpu_time()
         for collector in self.collectors:
             collector.start()
         return self
 
     def __exit__(self, *args):
+        self.end()
+
+    def end(self):
+        if self.done:
+            return
+        self.done = True
         try:
             for collector in self.collectors:
                 collector.stop()
-            self.duration = time.time() - self.start_time
+            self.duration = real_time() - self.start_time
+            self.cpu_duration = real_cpu_time() - self.start_cpu_time
             self._add_file_lines(self.init_stack_trace)
 
             if self.db:
@@ -530,26 +627,41 @@ class Profiler:
                     values = {
                         "name": self.description,
                         "session": self.profile_session,
-                        "create_date": datetime.datetime.now(),
+                        "create_date": real_datetime_now(),
                         "init_stack_trace": json.dumps(_format_stack(self.init_stack_trace)),
                         "duration": self.duration,
+                        "cpu_duration": self.cpu_duration,
                         "entry_count": self.entry_count(),
+                        "sql_count": sum(len(collector.entries) for collector in self.collectors if collector.name == 'sql')
                     }
+                    others = {}
                     for collector in self.collectors:
                         if collector.entries:
-                            values[collector.name] = json.dumps(collector.entries)
-                    query = sql.SQL("INSERT INTO {}({}) VALUES %s RETURNING id").format(
-                        sql.Identifier("ir_profile"),
-                        sql.SQL(",").join(map(sql.Identifier, values)),
+                            if collector._store == "others":
+                                others[collector.name] = json.dumps(collector.entries)
+                            else:
+                                values[collector.name] = json.dumps(collector.entries)
+                    if others:
+                        values['others'] = json.dumps(others)
+                    query = SQL(
+                        "INSERT INTO ir_profile(%s) VALUES %s RETURNING id",
+                        SQL(",").join(map(SQL.identifier, values)),
+                        tuple(values.values()),
                     )
-                    cr.execute(query, [tuple(values.values())])
-                    profile_id = cr.fetchone()[0]
-                    _logger.info('ir_profile %s (%s) created', profile_id, self.profile_session)
+                    cr.execute(query)
+                    self.profile_id = cr.fetchone()[0]
+                    _logger.info('ir_profile %s (%s) created', self.profile_id, self.profile_session)
+        except OperationalError:
+            _logger.exception("Could not save profile in database")
         finally:
-            if self.disable_gc:
-                gc.enable()
+            self.exit_stack.close()
             if self.params:
                 del self.init_thread.profiler_params
+            if self.log:
+                _logger.info(self.summary())
+
+    def _get_cm_proxy(self):
+        return Nested(self)
 
     def _add_file_lines(self, stack):
         for index, frame in enumerate(stack):
@@ -557,6 +669,8 @@ class Profiler:
             if line != '':
                 continue
             # retrieve file lines from the filecache
+            if not lineno:
+                continue
             try:
                 filelines = self.filecache[filename]
             except KeyError:
@@ -581,7 +695,7 @@ class Profiler:
         This is mainly useful to uniquify a path between executions.
         """
         return path.format(
-            time=datetime.datetime.now().strftime("%Y%m%d-%H%M%S"),
+            time=real_datetime_now().strftime("%Y%m%d-%H%M%S"),
             len=self.entry_count(),
             desc=re.sub("[^0-9a-zA-Z-]+", "_", self.description)
         )
@@ -601,12 +715,19 @@ class Profiler:
         return json.dumps({
             "name": self.description,
             "session": self.profile_session,
-            "create_date": datetime.datetime.now().strftime("%Y%m%d-%H%M%S"),
+            "create_date": real_datetime_now().strftime("%Y%m%d-%H%M%S"),
             "init_stack_trace": _format_stack(self.init_stack_trace),
             "duration": self.duration,
             "collectors": {collector.name: collector.entries for collector in self.collectors},
         }, indent=4)
 
+    def summary(self):
+        result = ''
+        for profiler in [self, *self.sub_profilers]:
+            for collector in profiler.collectors:
+                result += f'\n{self.description}\n{collector.summary()}'
+        return result
+
 
 class Nested:
     """
@@ -619,16 +740,16 @@ class Nested:
     be ignored, too. This is also why Nested() does not use
     contextlib.contextmanager.
     """
-    def __init__(self, profiler, context_manager):
-        self.profiler = profiler
-        self.context_manager = context_manager
+    def __init__(self, profiler, context_manager=None):
+        self._profiler__ = profiler
+        self.context_manager = context_manager or nullcontext()
 
     def __enter__(self):
-        self.profiler.__enter__()
+        self._profiler__.__enter__()
         return self.context_manager.__enter__()
 
     def __exit__(self, exc_type, exc_value, traceback):
         try:
             return self.context_manager.__exit__(exc_type, exc_value, traceback)
         finally:
-            self.profiler.__exit__(exc_type, exc_value, traceback)
+            self._profiler__.__exit__(exc_type, exc_value, traceback)
